package twcsckernel.serverKernel.impl;

import java.rmi.RemoteException;
import java.rmi.server.ServerNotActiveException;

import twcsckernel.projectbase.common.RemoteServer;
import twcsckernel.projectbase.common.User;
import twcsckernel.serverKernel.io.LogWriter;
import twcsckernel.serverKernel.utils.RmiManager;
import twcsckernel.serverKernel.utils.ServerGlobalState;
import twcsckernel.serverKernel.utils.UserData;

/**
 * Implementacja serwera logowania 
 * {@link twcsckernel.projectbase.common.RemoteServer RemoteServer}.
 * @author VMD Group
 * 
 */
public class RemoteServerImpl implements RemoteServer {

	private ServerGlobalState globalState;

	private RmiManager rmiManager;

	public RemoteServerImpl(ServerGlobalState globalState, RmiManager rmiMan) {
		this.globalState = globalState;
		rmiManager = rmiMan;
	}

	public User login(String login, String password) throws RemoteException {
		String userHost;
		;
		try {
			userHost = java.rmi.server.RemoteServer.getClientHost();
		} catch (ServerNotActiveException e1) {
			userHost = "unknown";
		}
		if (globalState.registeredUsers.checkPassword(login, password)) {
			UserData myself = globalState.registeredUsers.getUserData(login);

			UserImpl newUser;
			try {
				newUser = new UserImpl(myself, globalState, rmiManager,
						userHost);
			} catch (RemoteException re) {
				throw re;
			} catch (Exception e) {
				throw new RemoteException("exception: "
						+ e.getClass().getName() + " message: "
						+ e.getMessage(),e.getCause());
			}
			rmiManager.exportObject(newUser);
			globalState.logWriter.writeUserLog("User: \"" + login + "\" ("
					+ userHost + ") successfully logged in.",
					LogWriter.USER_LOGIN);
			return newUser;
		} else {
			globalState.logWriter.writeErrorLog(
					"Incorrect login/password attempt for user: \"" + login
							+ "\" (" + userHost + ")",
					LogWriter.ERROR_LOGIN_PASSWORD);
			return null;
		}
	}

}
