package twcsckernel.serverKernel.io;

import twcsckernel.projectbase.common.FsChangeListener;
import twcsckernel.projectbase.io.LocalFsChangeListener;
import twcsckernel.projectbase.io.UserRootPathManager;
import twcsckernel.serverKernel.impl.UserImpl;

/**
 * Interfejs wpdzielonego listenera zmian w systemie plikw. Przechowuje
 * listenery zdalne zgoszone przez klientw zdalnych. Jest on przekazywany do
 * fabryk plikw poszczeglnych uytkownikw (nie za same listenery zdalne) i
 * peni rol porednika (proxy), przez co mona realizowa w rny sposb
 * polityk wymiany zdarze pomidzy rnymi uytkownikami. Fakt, e dziedziczy
 * implementacj <i>Remote</i> jest bez znaczenia.
 * 
 * @author VMD Group
 * 
 */
public interface SharedFsChangeListener extends LocalFsChangeListener {

	/**
	 * Metoda pozwala na dodanie listenera, ktry naley do podanego
	 * uytkownika. Parametr <i>rootManager</i> jest managerem cieki root
	 * zastosowanym dla zdarzen tego listenera. Przekazywane zdarzenia posiadaj
	 * globalne cieki do plikw i tumaczenie na cieki wzgldne uytkownika
	 * ley w kompetencji tej klasy wanie przy pomocy przekazanego root
	 * managera.
	 * 
	 * @param fsChL -
	 *            dodawany listener
	 * @param owner -
	 *            waciciel listenera
	 * @param rootManager -
	 *            manager cieki root zastosowany dla zdarzenia
	 */
	public void addFsChangeListener(FsChangeListener fsChL, UserImpl owner,
			UserRootPathManager rootManager);

	/**
	 * Metoda pozwala na rczne usunicie podanego listenera.
	 * 
	 * @param fsChL -
	 *            listener do usunicia
	 * @param owner -
	 *            waciciel listenera
	 * @return - <code>true</code> jeli listener by zarejestrowany i zosta
	 *         usunity lub <code>false</code> w przeciwnym wypadku
	 */
	public boolean removeFsChangeListener(FsChangeListener fsChL, UserImpl owner);

	/**
	 * Metoda pozwala usun listenery zdalne danego uytkownika (potrzebne np.
	 * przy wologowaniu uytkownika).
	 * 
	 * @param owner -
	 *            uytkownik
	 */
	public void removeUserListeners(UserImpl owner);

}
