package twcsckernel.serverKernel.plugins;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * Uchwyt plugina na serwerze zawierajcy komplet informacji tj.: deskryptor,
 * cieki do plikw JAR klienta i serwera oraz instancj PluginBuildera
 * (o ile plugina pracuje w trybie zdalnymm czyli deskryptor.isRemote==true).
 * 
 * @author VMD Group
 * 
 */
public class PluginHandle {

	/**
	 * Deskryptor plugina.
	 */
	public final PluginDescriptor descriptor;

	private final String pluginPath;

	private final String serverFilename;

	private final String clientFilename;
	
	private final byte[] clientPluginData;

	/**
	 * klasa serwerowej czci plugina - jeli plugin nie jest przeznaczony do
	 * pracy zdalnej, to wartoci jest <code>null</code>.
	 */
	public final PluginBuilder serverPluginBuilder;

	/**
	 * @return cieka do JARa serwerowej czci plugina - jeli plugin nie
	 *         posiada czci serwerowej, to wartoci jest <code>null</code>.
	 */
	public String getServerJarPath() {
		if (descriptor.isRemote)
			return pluginPath + "/" + serverFilename;
		else
			return null;
	}

	/**
	 * @return cieka do JARa klienckiej czci plugina.
	 */
	public String getClientJarPath() {
		return pluginPath + "/" + clientFilename;
	}
	
	/**
	 * Metoda zwraca binarn posta pliku JAR klienckiej czci plugina.
	 * @return - binarna posta pliku JAR klienta
	 */
	public byte[] getClientPluginData() {
		/*
		 * Mona dla bezpieczestwa zrobi:
		 * return clientPluginData.clone();
		 * ale to mogoby by zbyt pamicioerne.
		 * */
		return clientPluginData;
	}

	public PluginHandle(PluginBuilder serverPluginBuilder,
			PluginDescriptor descriptor, String pluginPath,
			String serverFilename, String clientFilename, byte[] clientPluginData) {
		this.clientFilename = clientFilename;
		this.descriptor = descriptor;
		this.pluginPath = pluginPath;
		this.serverFilename = serverFilename;
		this.serverPluginBuilder = serverPluginBuilder;
		this.clientPluginData=clientPluginData;
	}

}
