package twcsckernel.serverKernel.plugins;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * PluginContainer jest kontenerem klas pluginw dostpnych dla danego typu
 * uytkownika.
 * 
 * @author VMD Group
 * 
 */
public class PluginHandleContainer {
	Map<String, Set<PluginHandle>> pluginTypeMap;

	Map<String, PluginHandle> clientPluginPkgMap;

	public PluginHandleContainer() {
		pluginTypeMap = new HashMap<String, Set<PluginHandle>>();
		clientPluginPkgMap = new HashMap<String, PluginHandle>();
	}

	/**
	 * Usuwa typ uytkownika.
	 * 
	 * @param userType
	 *            typ uytkownika
	 * @return true gdy usunito, false gdy nie znaleziono takiej nazwy typu
	 */
	public boolean removeUserType(String userType) {
		if (pluginTypeMap.get(userType) == null)
			return false;
		pluginTypeMap.remove(userType);
		return true;
	}

	/**
	 * Dodaje klas pluginu do listy dostpnych dla danego typu uytkownika.
	 * 
	 * @param userType -
	 *            typ uytkoniwka
	 * @param plugin -
	 *            klasa pluginu
	 * @return - <i>true</i> jeli udao si doda plugin lub
	 *         <code>false</code> w przeciwnym wypadku
	 */
	public boolean addPluginToUserType(String userType, PluginHandle plugin) {
		Set<PluginHandle> userSet = pluginTypeMap.get(userType);
		if (userSet == null) {
			userSet = new HashSet<PluginHandle>();
			pluginTypeMap.put(userType, userSet);
		}

		if (clientPluginPkgMap.get(plugin.descriptor.clientPackagePath) == null)
			clientPluginPkgMap.put(plugin.descriptor.clientPackagePath, plugin);

		return userSet.add(plugin);
	}

	/**
	 * Zwraca tablic uchwytw pluginw dostpnych dla danego typu uytkownika.
	 * 
	 * @param userType -
	 *            nazwa typu uytkownika
	 * @return talica klas pluginw (lub <code>null</code> jeli nie ma
	 *         takiego typu uytkownika)
	 */
	public PluginHandle[] getUserTypePlugins(String userType) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		if (userPluginSet == null)
			return null;
		return userPluginSet.toArray(new PluginHandle[0]);
	}

	/**
	 * Metoda zwraca tablic deskryptorw pluginw dla danego typu uytkownika
	 * 
	 * @param userType -
	 *            typ uytkownika
	 * @return - talibca deskryptorw pluginw lub <code>null</code> jeli
	 *         podany typ nie istnieje
	 */
	public PluginDescriptor[] getUserPluginsDescriptors(String userType) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		if (userPluginSet == null)
			return new PluginDescriptor[0];
		PluginDescriptor[] descriptors = new PluginDescriptor[userPluginSet
				.size()];
		int index = 0;
		for (PluginHandle pluginHandle : userPluginSet) {
			descriptors[index] = pluginHandle.descriptor;
			index++;
		}
		return descriptors;
	}

	/**
	 * Metoda zwraca uchwyt plugina danego typu uytkownika o wyspecyfikowanej
	 * ciece pakietowej czci klienckiej.
	 * 
	 * @param userType -
	 *            typ uytkownika
	 * @param clientPackagePath -
	 *            cieka pakietowa klienckiej czci plugina
	 * @return - uchwyt szukanego plugina lub <code>null</code> jeli nie
	 *         istnieje typ uytkownika lub plugin o podanej ciece
	 */
	public PluginHandle getUserPluginByClientPackagePath(String userType,
			String clientPackagePath) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		PluginHandle selectedPlugin = clientPluginPkgMap.get(clientPackagePath);
		if (userPluginSet.contains(selectedPlugin))
			return selectedPlugin;
		return null;

	}
}
