package twcsckernel.serverKernel.usr;

import java.io.IOException;

import twcsckernel.projectbase.io.FileSecurityManager;
import twcsckernel.serverKernel.impl.UserImpl;
import twcsckernel.serverKernel.io.SharedFsChangeListener;
import twcsckernel.serverKernel.utils.UserData;

/**
 * Klasa jest kontenerem zawierajcym list aktualnie zalogowanych uytkownikw
 * oraz suy zarzdzaniu listenerami systemu plikw (mog by wspdzielone).
 * Uytkownik przy logowaniu pobiera listener metod
 * {@link #getSharedUserListener(UserImpl) getSharedUserListener}, ktry
 * przekazuje swoim fabrykom plikw. Dodawanie i usuwanie listenerw zdalnych
 * ktre klient przekae nastpuje wanie poprzez ten listener.
 * 
 * @author VMD Group
 * 
 */
public interface LoggedUsersManager {

	/**
	 * Metoda logujca danego uytkownika.
	 * 
	 * @param user -
	 *            logowany uytkownik
	 * @return - <code>FileSecurityManager</code>
	 * @throws LoginException -
	 *             wyjtek rzucany jeli uytkowik prbuje si zalogowa
	 *             ponownie lub wystpi bd podczas tworzenia managera
	 *             bezpieczestwa
	 */
	public FileSecurityManager login(UserImpl user) throws LoginException;

	/**
	 * Metoda wylogowania danego uytkownika.
	 * 
	 * @param user -
	 *            uytkownik do wylogowania
	 * @return - <code>true</code> jeli uytkownik by zalogowany i zosta
	 *         wylogowany lub <code>false</code> w przeciwnym wypadku
	 */
	public boolean logout(UserImpl user);

	/**
	 * @return - liczba wszystkich zalogowanych uytkownikw
	 */
	public int getAllUsersNumber();

	/**
	 * @param login -
	 *            login uytkownika
	 * @return - liczba zaogowanych uytkownikw o podanym loginie
	 */
	public int getLoginUsersNumber(String login);
	
	/**
	 * @return - zalogowani uytkownicy o podanym loginie
	 */
	public UserImpl[] getLoggedUserType(String login);

	/**
	 * @return - wszyscy zalogowani uytkownicy
	 */
	public UserImpl[] getAllLoggedUsers();

	/**
	 * Dodanie listenera logowania/wylogowania uytkownikw. Parametr
	 * <i>UserImpl owner</i> jest potrzebny identyfikacji listenerw ktre maj
	 * by automatycznie zwolnione podczas wylogowania ich waciciela.
	 * 
	 * @param loginListener -
	 *            listener logowania/wylogowania
	 * @param owner -
	 *            waciciel listenera
	 */
	public void addLoginListener(LoginListener loginListener, UserImpl owner);

	/**
	 * Metoda pozwala na rczne wyrejestrowanie listenera logowania
	 * 
	 * @param loginListener -
	 *            listener logowania/wylogowania
	 * @param owner -
	 *            waciciel listenera (optymalizacja)
	 * @return - <code>true</code> jeli listener by zarejestrowany i udao
	 *         si go wyrejestrowa lub <code>false</code> w przeciwnym
	 *         wypadku
	 */
	public boolean removeLoginListener(LoginListener loginListener,
			UserImpl owner);

	/**
	 * Metoda suca do pobierania listenera uytkownika, ktry bdzie
	 * przekazany dalej do fabryki plikw. Dla tej samej instancji uytkownika
	 * metoda powinna zwrci ten sam listener (w przypaku odwoania do tej
	 * metody przez pluginy).
	 * 
	 * @param user -
	 *            uytkonik dla ktrego listener ma by przydzielony/przekazany
	 * @return - listener
	 */
	public SharedFsChangeListener getSharedUserListener(UserImpl user);

	/**
	 * Metoda tworzy/pobiera manager bezpieczestwa dla podanego uytkownika.
	 * 
	 * @param userData -
	 *            deskryptor uytkownika
	 * @return - manager bezpieczestwa przydzielony temu uytkownikowi
	 * @throws IOException -
	 *             wyjtek rzucany w przypadku bdu utworzenia managera dla
	 *             podanego deskryptora
	 */
	public FileSecurityManager createSecurityManager(UserData userData)
			throws IOException;

}
