package twcsckernel.serverKernel.utils;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

import twcsckernel.serverKernel.io.TypeIO;

/**
 * Kontener zarejestrowanych typw uytkownikw wraz z zarzdzaniem.
 * 
 * @author VMD Group
 * 
 */
public class RegisteredTypes {
	private Map<String, TypeData> types = null;

	TypeIO typeIO;

	/**
	 * Konstruktor ustawia podany manager I/O i odczytuje wszystkie typy.
	 * 
	 * @param typeIO -
	 *            obiekt zarzdzajcy operacjami I/O na typach
	 * @throws IOException -
	 *             bd odczytu
	 */
	public RegisteredTypes(TypeIO typeIO) throws IOException {
		this.typeIO = typeIO;
		types = Collections.synchronizedMap(typeIO.readTypes());
	}

	/**
	 * Metoda dodaje typ uytkownika do zbioru danych zarzdzanego przez podany
	 * w konstruktorze obiekt I/O
	 * 
	 * @param data -
	 *            dane typu do dodania
	 * @return - <code>true</code> jeli operacja si powioda (nie byo
	 *         wczeniej typu o podanej nazwie) lub <code>false</code> w
	 *         przeciwnym wypadku
	 * @throws IOException -
	 *             bd zapisu
	 */
	public boolean addNewType(TypeData data) throws IOException {
		if (!types.containsKey(data.getTypeName()))
			return typeIO.addNewType(data);
		return false;
	}

	/**
	 * Metoda usuwa typ uytkownika ze zbioru danych zarzdzanego przez podany w
	 * konstruktorze obiekt I/O
	 * 
	 * @param name -
	 *            nazwa typu do usunicia
	 * @return - <code>true</code> jeli operacja si powioda lub
	 *         <code>false</code> w przeciwnym wypadku
	 * @throws IOException -
	 *             bd zapisu
	 */
	public boolean removeType(String name) throws IOException {
		return typeIO.removeType(name, types);
	}

	/**
	 * @return - tablica wszystkich typw
	 */
	public TypeData[] getRegisteredTypes() {
		return types.values().toArray(new TypeData[0]);
	}

	/**
	 * @return - zbir wszystkich nazw typw
	 */
	public Set<String> getTypeNamesSet() {
		return types.keySet();
	}

}
