package twcsckernel.serverKernel.utils;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import twcsckernel.serverKernel.io.UserIO;

/**
 * Kontener zarejestrowanych uytkownikw wraz z zarzdzaniem.
 * 
 * @author VMD Group
 * 
 */
public class RegisteredUsers {
	Map<String, UserData> logins = null;

	UserIO userIO;

	/**
	 * Konstruktor ustawia podany manager I/O i odczytuje wszystkich
	 * uytkownikw.
	 * 
	 * @param userIO -
	 *            obiekt zarzdzajcy operacjami I/O uytkownikw
	 * @throws IOException -
	 *             bd odczytu
	 */
	public RegisteredUsers(UserIO userIO) throws IOException {
		this.userIO = userIO;
		logins = Collections.synchronizedMap(userIO.readUsers());
	}

	/**
	 * Metoda sprawdza czy podane haso w postaci jawnej zgadza si z
	 * zahaszowanym hasem z bazy uytkownikw.
	 * 
	 * @param login -
	 *            login uytkownika
	 * @param passwd -
	 *            haso uytkownika
	 * @return - <code>true</code> jeli haso zostao poprawnie zweryfikowane
	 *         lub <code>false</code> w przeciwnym wypadku
	 */
	public boolean checkPassword(String login, String passwd) {
		if (logins.get(login) != null) {
			UserData userData = logins.get(login);
			if (Encoder.hashPassword(passwd)
					.equals(userData.getEncodedPasswd()))
				return true;
		}
		return false;

	}

	/**
	 * Metoda dodaje nowego uytkownika do bazy zarejestrowanych uytkownikw.
	 * 
	 * @param data -
	 *            dane uytkownika do dodania
	 * @return - <code>true</code> jeli operacja si powioda (nie byo
	 *         takiego uytkonika) lub <code>false</code> w przeciwnym wypadku
	 * @throws IOException -
	 *             bd zapisu
	 */
	public boolean addNewUser(UserData data) throws IOException {
		if (!logins.containsKey(data.getLogin()))
			return userIO.addNewUser(data);
		return false;
	}

	/**
	 * Metoda usuwa uytkownika o podanym loginie z bazy.
	 * 
	 * @param login -
	 *            login uytkownika do usunicia
	 * @return - <code>true</code> jeli operacja si powioda lub
	 *         <code>false</code> w przeciwnym wypadku
	 * @throws IOException - bd zapisu
	 */
	public boolean removeUser(String login) throws IOException {
		return userIO.removeUser(login, logins);
	}

	/**
	 * @return - talibca zarejestrowanych uytkownikw
	 */
	public UserData[] getRegisteredUsers() {
		return logins.values().toArray(new UserData[0]);
	}

	/**
	 * @param login - login uytkownika
	 * @return - dane uytkownika o wskazanym loginie
	 */
	public UserData getUserData(String login) {
		return logins.get(login);
	}

}
