package twcsckernel.projectbase.common;

import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import twcsckernel.projectbase.io.FileDescriptor;
import twcsckernel.projectbase.io.ItemNotFoundException;

/**
 * Interfejs zdalny zawierajcy metody analogiczne do metod klasy
 * <i>java.io.File</i>. Sposb ich dziaania, parametry, wyniki oraz wyjtki s
 * rwnie analogiczne do metod klasy File. Wsplne rnice midzy metodami z
 * klasy File a metodami tego interfejsu s nastpujce:
 * <ul>
 * <li>kada metoda odwouje si do pliku, ktry jest utworzony po stronie
 * serwerowej i przechowywany w mapie, ktrej kluczem jest identyfikator typu
 * Integer.Pierwszym parametrem kadej metody jest zatem ten identyfikator,</li>
 * <li>jesli metoda klasy File rzuca wyjatek
 * <i>SecurityException,ItemNotFoundException</i>, to analogiczne metody
 * rzucaj wyjtek <i>RMISecurityException,ItemNotFoundException</i>,</li>
 * <li>sciezka absolutna/kanoniczna jest podawana od katalogu root wskazanego w
 * implementacji tego interfejsu (powinien by to katalog root zalogowanego
 * uytkownika)</li>
 * </ul>
 * 
 * @author VMD Group
 * 
 */
public interface FileFactory extends Remote {
	public boolean disposeFileHandle(int fileID) throws RemoteException,
			ItemNotFoundException;

	public char getSeparatorChar() throws RemoteException;

	public String getSeparator() throws RemoteException;

	public char getPathSeparatorChar() throws RemoteException;

	public String getPathSeparator() throws RemoteException;

	public String getName(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public String getParent(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public String getPath(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean canRead(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean canWrite(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean exists(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean isDirectory(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean isFile(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean isHidden(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public long lastModified(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public long length(int fileID) throws RemoteException, SecurityException,
			ItemNotFoundException;

	public boolean createNewFile(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException, IOException;

	public boolean delete(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public FileDescriptor[] list(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean mkdir(int fileID) throws RemoteException, SecurityException,
			ItemNotFoundException;

	public boolean mkdirs(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean renameTo(int fileID, String dest) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public boolean setLastModified(int fileID, long time)
			throws RemoteException, SecurityException, ItemNotFoundException;

	public boolean setReadOnly(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;

	public FileDescriptor getDescriptor(int fileID) throws RemoteException,
			SecurityException, ItemNotFoundException;
}
