/**
 * 
 */
package twcsckernel.projectbase.common;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import twcsckernel.projectbase.io.RemoteFile;
import twcsckernel.projectbase.io.RemoteFileInputStream;
import twcsckernel.projectbase.io.RemoteFileOutputStream;

public interface RemoteFileSystem extends Remote {

	/**
	 * Tworzy nowy obiekt pliku zdalnego na serwerze o podanej ciece.
	 * 
	 * @param path
	 *            cieka pliku zdalnego na serwerze (liczc od katalogu root
	 *            uytkownika)
	 * @return obiekt pliku zdalnego
	 * @throws RemoteException
	 * @throws IOException -
	 *             wyjtek jest rzucany jeli serwer nie moe utworzy cieki
	 *             kanonicznej dla podanej cieki
	 * @throws SecurityException -
	 *             prba dostpu do zabronionego lub nieprawidowego pliku
	 */
	public RemoteFile newRemoteFile(String path) throws RemoteException,
			SecurityException, IOException;

	/**
	 * Metoda tworzy nowy zdalny strumie wyjcia.
	 * 
	 * @param filePath -
	 *            cieka pliku na serwerze sprawdzana przez odpowiedni manager
	 *            bezpieczestwa uytkownika
	 * @return - zdalny strumie wyjcia
	 * @throws RemoteException -
	 *             standardowy wyjtek zdalny
	 * @throws FileNotFoundException -
	 *             nie odnaleziono pliku (podana ciezka jest katalogiem)
	 * @throws SecurityException -
	 *             prba dostpu do zabronionego lub nieprawidowego pliku
	 */
	public RemoteFileOutputStream newFileOutputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException;

	/**
	 * Metoda tworzy nowy zdalny strumie wejcia.
	 * 
	 * @param filePath -
	 *            cieka pliku na serwerze sprawdzana przez odpowiedni manager
	 *            bezpieczestwa uytkownika
	 * @return - zdalny strumie wejcia
	 * @throws RemoteException -
	 *             standardowy wyjtek zdalny
	 * @throws FileNotFoundException -
	 *             nie odnaleziono pliku
	 * @throws SecurityException -
	 *             prba dostpu do zabronionego lub nieprawidowego
	 *             pliku/katalogu
	 */
	public RemoteFileInputStream newFileInputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException;
}
