package twcsckernel.projectbase.common;

import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * Interfejs zdalny uytkownika.
 * 
 * @author VMD Group
 * 
 */
public interface User extends Remote {

	/**
	 * Listowanie dostpnych pluginw dla klienta
	 * 
	 * @return tablica deskryptorw dostpnych pluginw
	 * @throws RemoteException
	 */
	public PluginDescriptor[] listClientPlugins() throws RemoteException;

	/**
	 * Download klienckiego JARa plugina podanej nazwie pakietowej.
	 * 
	 * @param clientPackagePath -
	 *            nazwa pakietowa klienckiej czci plugina
	 * @return - tablica bajtw pliku JAR lub <code>null</code> jeli nie
	 *         znaleziono plugina o podanej nazwie
	 * @throws RemoteException
	 * @throws IOException -
	 *             wyjtek rzucany gdy wystpi wyjtek podczas odczytu plugina
	 *             z dysku
	 */
	public byte[] downloadClientPlugin(String clientPackagePath)
			throws RemoteException, IOException;

	/**
	 * Pobranie agenta sieciowego zdalnej czci plugina o podanej klienckiej
	 * ciece pakietowej.
	 * 
	 * @param clientPackagePath -
	 *            nazwa pakietowa klienckiej czci plugina
	 * @param clientCallback -
	 *            wasny agent sieciowy
	 * @return - agent serwerowej czci plugina
	 */
	public RemoteAgent getRemotePluginAgent(String clientPackagePath,
			RemoteAgent clientCallback) throws RemoteException;

	/**
	 * Pobiera fabryk plikw danego uytkownika.
	 * 
	 * @return - fabryka plikw
	 * @throws RemoteException
	 */
	public RemoteFileSystem getUserFileSystem() throws RemoteException;

	/**
	 * Ustawia na serwerze listener zmian w systemie plikw.
	 * 
	 * @param fsChangeListener -
	 *            listener zmian w systemie plikw
	 * @throws RemoteException
	 */
	public void setFsChangeListener(FsChangeListener fsChangeListener)
			throws RemoteException;

	/**
	 * Wylogowanie uytkownika z serwera.
	 * 
	 * @throws RemoteException
	 */
	public void logout() throws RemoteException;

	/**
	 * Metoda suy do pingowania serwera w celu utrzymania aktywnego
	 * rejestracji uytkownika w serwerze RMI.
	 * 
	 * @throws RemoteException
	 */
	public void ping() throws RemoteException;

	/**
	 * Metoda zwraca maksymalny czas przez jaki uytkownik moe nie wykonywa
	 * adnej metody na obiekcie <i>User</i>. Po przekroczeniu tego czasu
	 * serwer ma prawo wylogowa klienta.
	 * 
	 * @return - maksymalny czas nieaktywnoci w milisekundach
	 * @throws RemoteException
	 */
	public long getActivityTimeout() throws RemoteException;
}
