package twcsckernel.projectbase.io;

import java.io.File;
import java.io.IOException;

/**
 * Interfejs mededera bezpieczestwa plikw, ktry sprawdza czy podana zdalnie
 * cieka jest poprawna i czy uytkownik ma do niej prawa.
 * 
 * @author VMD Group
 * 
 */
public interface FileSecurityManager {
	public static final int ACC_OK = 0;

	public static final int ACC_NOT_FOUND = 1;

	public static final int ACC_ILLEGAL = 2;

	/**
	 * Sprawdza, czy uytkownik ma prawa do podanej cieki (kanoniczna cieka
	 * <i>root path</i> uytkownika zawiera si w kanonicznej ciece obiektu
	 * file) oraz jeli prawa s sprawdza czy podany plik istnieje.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - s prawa oraz plik istnieje
	 *            <li><i>ACC_NOT_FOUND</i> - s prawa, ale plik nie istnieje
	 *            lub niepoprawna cieka (w ramach praw)
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna cieka
	 *            </ul>
	 */
	public int checkReadAccess(File file);

	/**
	 * Sprawdza, czy uytkownik ma prawa do pliku o podanej ciece (kanoniczna
	 * cieka <i>root path</i> uytkownika zawiera si w kanonicznej ciece
	 * obiektu file) oraz jeli prawa s sprawdza czy podany plik jest plikiem
	 * poprawnym systemu plikw.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - s prawa oraz nazwa pliku jest poprawna
	 *            <li><i>ACC_NOT_FOUND</i> - s prawa, ale nazwa pliku nie
	 *            jest poprawna lub niepoprawna cieka (w ramach praw)
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna cieka
	 *            </ul>
	 */
	public int checkWriteAccess(File file);

	/**
	 * Sprawdza, czy uytkownik ma prawa do cieki podanego pliku (kanoniczna
	 * cieka <i>root path</i> uytkownika zawiera si w kanonicznej ciece
	 * obiektu file) - uywana przez
	 * {@link #checkReadAccess(File) checkReadAccess} oraz
	 * {@link #checkWriteAccess(File) checkWriteAccess}.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - s prawa = root path uytkownika zawiera
	 *            si w filePath
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna cieka
	 *            </ul>
	 */
	public int checkRootPath(File file);

	/**
	 * @return meneder cieki root praw uytkownika
	 */
	public UserRootPathManager getUserRootPathManager();

	/**
	 * Ustawienie cieki root security managera
	 * 
	 * @param userRootPath -
	 *            cieka root uytkownika
	 * @throws IOException -
	 *             ew. wyjtek rzucany przy pobieraniu cieki kanonicznej pliku
	 */
	public void setRootPath(String userRootPath) throws IOException;

}
