/**
 * 
 */
package twcsckernel.projectbase.io;

import java.io.Serializable;
import java.rmi.RemoteException;

import twcsckernel.projectbase.common.FileFactory;

/**
 * @author VMD Group
 * 
 */
public class RemoteFile implements Serializable {

    private static final long serialVersionUID = 1L;

    private final int fileID;

    private final FileFactory rff;

    RemoteFile(int fileID, FileFactory rff)
            throws NullPointerException {
        if (rff == null)
            throw new NullPointerException("RemoteFileFactory is null");
        this.fileID = fileID;
        this.rff = rff;
    }

    public boolean disposeFileHandle() {
        try {
            return rff.disposeFileHandle(fileID);
        } catch (RemoteException e) {
            return false;
        } catch (ItemNotFoundException e) {
            return false;
        }
    }

    public Character getSeparatorChar() {
        try {
            return rff.getSeparatorChar();
        } catch (RemoteException e) {
            return null;
        }
    }

    public String getSeparator() {
        try {
            return rff.getSeparator();
        } catch (RemoteException e) {
            return null;
        }

    }

    public Character getPathSeparatorChar() {
        try {
            return rff.getPathSeparatorChar();
        } catch (RemoteException e) {
            return null;
        }
    }

    public String getPathSeparator() throws SecurityException {
        try {
            return rff.getPathSeparator();
        } catch (RemoteException e) {
            return null;
        }
    }

    public String getName() throws SecurityException {
        try {
            return rff.getName(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public String getParent() throws SecurityException {
        try {
            return rff.getParent(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public String getPath() throws SecurityException {
        try {
            return rff.getPath(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean canRead() throws SecurityException {
        try {
            return rff.canRead(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean canWrite() throws SecurityException {
        try {
            return rff.canWrite(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean exists() throws SecurityException {
        try {
            return rff.exists(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean isDirectory() throws SecurityException {
        try {
            return rff.isDirectory(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean isFile() throws SecurityException {
        try {
            return rff.isFile(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean isHidden() throws SecurityException {
        try {
            return rff.isHidden(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Long lastModified() throws SecurityException {
        try {
            return rff.lastModified(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Long length() throws SecurityException {
        try {
            return rff.length(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean createNewFile() throws SecurityException {
        try {
            return rff.createNewFile(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (Exception infe) {
            return null;
        } 
    }

    public Boolean delete() throws SecurityException {
        try {
            return rff.delete(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public FileDescriptor[] list() throws SecurityException {
        try {
            return rff.list(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean mkdir() throws SecurityException {
        try {
            return rff.mkdir(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean mkdirs() throws SecurityException {
        try {
            return rff.mkdirs(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean renameTo(String dest) throws SecurityException {
        try {
            return rff.renameTo(fileID,dest);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean setLastModified(long time) throws SecurityException {
        try {
            return rff.setLastModified(fileID,time);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }

    public Boolean setReadOnly() throws SecurityException {
        try {
            return rff.setReadOnly(fileID);
        } catch (RemoteException e) {
            return null;
        } catch (ItemNotFoundException infe) {
            return null;
        }
    }
    
    public FileDescriptor getDescriptor() {
    	try {
			return rff.getDescriptor(fileID);
		} catch (Exception e) {
			return null;
		}
    }

}
