package twcsckernel.projectbase.io;

import java.io.IOException;
import java.io.OutputStream;
import java.io.Serializable;

import twcsckernel.projectbase.common.WriterFactory;

public class RemoteFileOutputStream extends OutputStream implements Serializable {
    
    private static final long serialVersionUID = 1L;

    private final WriterFactory wf;

    private final int streamID;
    private static final IOException streamNotFound=new IOException("Stream not found");

    public RemoteFileOutputStream(int streamID, WriterFactory wf) {
        this.streamID = streamID;
        this.wf = wf;
    }
    
    @Override
    public void write(int b) throws IOException {
        try {
            wf.write(streamID,b);
        } catch (ItemNotFoundException e) {
            throw streamNotFound;
        }
    }
    
    @Override
    public void write(byte b[]) throws IOException {
        write(b, 0, b.length);
    }
    
    @Override
    public void write(byte b[], int off, int len) throws IOException {
        if (b == null) {
            throw new NullPointerException();
        } else if ((off < 0) || (off > b.length) || (len < 0)
                || ((off + len) > b.length) || ((off + len) < 0)) {
            throw new IndexOutOfBoundsException();
        } else if (len == 0) {
            return;
        }
        byte[] writtenTable=null;
        if (off==0 && len==b.length)
            writtenTable=b;
        else {
            writtenTable=new byte[len];
            for (int i=0;i<len;i++)
                writtenTable[i]=b[off+i];
        }
        try {
            wf.write(streamID,writtenTable);
        } catch (ItemNotFoundException e) {
            throw streamNotFound;
        }

    }

    @Override
    public void flush() throws IOException {
        try {
            wf.flush(streamID);
        } catch (ItemNotFoundException e) {
            throw streamNotFound;
        }
    }

    @Override
    public void close() throws IOException {
        try {
            wf.close(streamID);
        } catch (ItemNotFoundException e) {
            throw streamNotFound;
        }
    }
}
