package twcsckernel.projectbase.io;

import java.io.File;
import java.io.IOException;
import java.util.regex.Pattern;

/**
 * Meneder cieki root uytkownika.
 * 
 * @author VMD Group
 * 
 */
public class UserRootPathManager {

	private String rootPath;

	private int rootPathLength;

	// wyr. reg. do zmieniania \ na /
	private static Pattern regex = Pattern.compile("\\\\");

	private static final String slash = "/";

	/**
	 * Konstruktor menedera cieki root. Podana cieka musi ustnie. cieka
	 * jest przeksztacana na posta kononiczn (absolutn i unikaln) i w
	 * takiej postaci bdzie uywana.
	 * 
	 * @param rootPath -
	 *            cieka root
	 * @throws IOException -
	 *             wyjtek rzucany w przypadku niepoprawnej cieki root
	 */
	public void setUserRootPath(String rootPath) throws IOException {
		File rootFile = new File(rootPath);
		if (!rootFile.exists())
			throw new IOException("Incorrect root path: " + rootPath);
		this.rootPath = rootFile.getCanonicalPath();
		rootPathLength = this.rootPath.length();
	}

	/**
	 * Metoda konwertuje ciek bezwzgldn systemu plikw na ciek
	 * bezwzgldn uytkownika. Separatorem w zwracanej ciece zawsze
	 * jest "/".
	 * 
	 * @param globalPath -
	 *            cieka bezwzgldna systemu plikw
	 * @return - cieka bezwzgldna uytkownika
	 */
	public String getUserAbsolutePath(String globalPath) {
		if (!globalPath.startsWith(rootPath))
			return null;
		String result = correctSlash(globalPath.substring(rootPathLength));
		if (!result.startsWith(slash))
			result = slash + result;
		return result;
	}

	private static String correctSlash(String root) {
		return regex.matcher(root).replaceAll(slash);
	}

	/**
	 * Metoda konwertuje ciek bezwzgldn systemu plikw na ciek
	 * bezwzgldn uytkownika.
	 * 
	 * @param file -
	 *            plik
	 * @return - cieka bezwzgldna uytkownika
	 */
	public String getUserAbsolutePath(File file) {
		return getUserAbsolutePath(file.getAbsolutePath());
	}

	/**
	 * Metoda konwertuje ciek bezwzgldn uytkownika na ciek bezwzgldn
	 * systemu plikw.
	 * 
	 * @param userPath -
	 *            cieka bezwzgldna uytkownika
	 * @return - cieka bezwzgldna systemu plikw
	 */
	public String getGlobalAbsolutePath(String userPath) {
		return correctSlash(rootPath + slash + userPath);
	}

	/**
	 * Metoda tworzy obiekt pliku majc do dyspozycji bezwzgldn ciek
	 * uytkownika.
	 * 
	 * @param userPath -
	 *            bzw. cieka uytkownika
	 * @return - obiekt pliku (java.io.File)
	 * @throws IOException -
	 *             wyjtek rzucany jeli niemoliwe jest uzyskanie kanonicznej
	 *             postaci pliku
	 */
	public File getFile(String userPath) throws IOException {

		return new File(getGlobalAbsolutePath(userPath)).getCanonicalFile();
	}

	/**
	 * @return zwraca ciek gwn uytkownika
	 */
	public String getRootPath() {
		return rootPath;
	}
}
