package twcsckernel.projectbase.utils;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * Klasa suy do dynamicznego rozszerzania cieki classpath (adowania
 * klas do systemowego classloadera).
 * @author VMD Group, internetowe grupy dyskusyjne
 *
 */
public class ClassPathLoader {
	private static final Class[] parameters = new Class[] { URL.class };

	/**
	 * Dodanie pliku JAR/katalogu do sciezki classpath.
	 * @param filePath - cieka do pliku JAR/katalogu 
	 * @throws IOException - bd podczas wczytywania
	 */
	public static void addFile(String filePath) throws IOException {
		addFile(new File(filePath));
	}

	/**
	 * Dodanie pliku JAR/katalogu do sciezki classpath.
	 * @param file - plik JAR/katalog 
	 * @throws IOException - bd podczas wczytywania
	 */
	public static void addFile(File file) throws IOException {
		addURL(file.toURL());
	}

	/**
	 * Dodanie pliku JAR/katalogu do sciezki classpath.
	 * @param url - URL pliku JAR/katalogu 
	 * @throws IOException - bd podczas wczytywania
	 */
	public static void addURL(URL url) throws IOException {
		URLClassLoader sysloader = (URLClassLoader) ClassLoader
				.getSystemClassLoader();
		Class sysclass = URLClassLoader.class;
		try {
			Method method = sysclass.getDeclaredMethod("addURL", parameters);
			method.setAccessible(true);
			method.invoke(sysloader, new Object[] { url });
		} catch (Exception e) {
			throw new IOException(
					"Error, could not add URL to system classloader:"
							+ e.getMessage());
		}
	}
}
