package twcsckernel.clientKernel.utils;

import java.rmi.RemoteException;
import java.util.Timer;
import java.util.TimerTask;

import twcsckernel.projectbase.common.User;

/**
 * Klasa odpowiedzialna za podtrzymanie polczenia, tak, eby user 
 * nie zosta wylogowany przez UserGarbageCollector na serwerze.
 * 
 * @author VMD Group
 * 
 */
public class ConnectionSustainer {

	private long interval;

	private Timer timer = null;
	private User user;
	
	private TimerTask gcTask = new TimerTask() {
	
		@Override
		public void run() {
			try {
				
				ConnectionSustainer.this.user.ping();
			} catch (RemoteException e) {
				disable();
				e.printStackTrace();
				System.exit(0);
				
			}catch (Exception e){
				disable();
				e.printStackTrace();
				System.exit(0);
			}
		}
	};

	

	/**
	 * Metoda ustawiajca obiekt uytkownika na serwerze na ktorym bedzie 
	 * wywoywana metoda <code>ping()</code>
	 * @param userObject
	 *            obiekt uytkownika na serwerze
	 */
	public synchronized void setUser(User userObject) {
		boolean ifEnabled = timer != null;
		disable();
		this.user = userObject;
		if (ifEnabled)
			enable();

	}

	/**
	 * Metoda wycza dziaanie Sustainera
	 */
	public synchronized void disable() {
		if (timer != null) {
			timer.cancel();
			timer = null;
		}
	}

	/**
	 * Metoda wcza dziaanie Sustainera o ile ustanowione s pola 
	 * <code>interval</code> i <code>user</code>
	 */
	public synchronized void enable() {
		System.out.println("Starting ping procedure...");
		if (user == null)
			return;
		if (interval == 0) {
			disable();
			return;
		}
		timer = new Timer(true);
		timer.schedule(gcTask, interval, interval);
	}

	/**
	 * Metoda ustawia interwa czasowy co ktry bedzie nastpowao wywoanie ping
	 * @param interval - interwa czasowy w milisekundach
	 */
	public synchronized void setInterval(long interval) {
		boolean ifEnabled = timer != null;
		disable();
		this.interval = interval;
		if (ifEnabled)
			enable();
	}

}
