package twcsckernel.clientKernel.io;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import twcsckernel.clientKernel.io.fsChangeListenerClasses.RegisteredListenersListElem;
import twcsckernel.clientKernel.utils.CommonVariablesContainer;
import twcsckernel.projectbase.common.FsChangeListener;
import twcsckernel.projectbase.io.FileDescriptor;
import twcsckernel.projectbase.io.FsChangeDescriptor;
import twcsckernel.projectbase.io.LocalFsChangeListener;
import twcsckernel.projectbase.io.RemoteFile;

public class FsChangeListenerServer implements FsChangeListener{
	/**
	 * 
	 */
	private static final long serialVersionUID = -2004662961257233974L;

	private Map<String, ArrayList<RegisteredListenersListElem>> registeredListings;

	private Map<LocalFsChangeListener, ArrayList<String>> registeredListeners;

	public void fsChangeNotify(FsChangeDescriptor fsDescriptor)
			throws RemoteException {

		// TODO: dla katalogów informowanie o zmianach w nich zawartych. Po
		// uzgodnieniu z bogiem listenerow

		String path;
		if (fsDescriptor.getChangeType() == FsChangeDescriptor.FILE_RENAME
				|| fsDescriptor.getChangeType() == FsChangeDescriptor.FILE_DELETE) {
			path = fsDescriptor.getSource();
		} else {

			path = fsDescriptor.getChangeInfo().getPath();
			if (path.length() == 0 || path.charAt(path.length()-1)!='/')
				path.concat("/");
			
			path = path + fsDescriptor.getChangeInfo().getName();
			
		}
//		System.out.println("Odebrano notify  type: "+fsDescriptor.getChangeType()+" na sciezke: "+path);
		
		synchronized (registeredListings) {
			ArrayList<RegisteredListenersListElem> tempElemList = registeredListings
					.get(path);
			if (tempElemList == null)
				return;
			for (RegisteredListenersListElem le : tempElemList) {
				if (le.typeListen(fsDescriptor.getChangeType())) {
					le.getListener().fsChangeNotify(fsDescriptor);
				}
			}
		}
		
		if (fsDescriptor.getChangeType()==FsChangeDescriptor.FILE_CREATE || fsDescriptor.getChangeType()==FsChangeDescriptor.FILE_DELETE){
			FsChangeDescriptor newDesc = new FsChangeDescriptor(FsChangeDescriptor.FILE_REFRESH);
			RemoteFile rf;
			FileDescriptor fd;
			try {
				rf = CommonVariablesContainer.rff.newRemoteFile(path);
				String parent  = rf.getParent();
				rf.disposeFileHandle();
				rf = CommonVariablesContainer.rff.newRemoteFile(parent);
				fd=rf.getDescriptor();
				rf.disposeFileHandle();
			} catch (RemoteException e) {
				// TODO Auto-generated catch block
				return;
			} catch (SecurityException e) {
				// TODO Auto-generated catch block
				return;
			} catch (IOException e) {
				// TODO Auto-generated catch block
				return;
			}
			
			newDesc.setChangeInfo(fd);
			fsChangeNotify(newDesc);
		}

	}

	public FsChangeListenerServer() {
		registeredListings = Collections
				.synchronizedMap(new HashMap<String, ArrayList<RegisteredListenersListElem>>());
		;
		registeredListeners = Collections
				.synchronizedMap(new HashMap<LocalFsChangeListener, ArrayList<String>>());
	}

	/**
	 * Metoda rejestruje obiekt mający otrzymywać informacje o zmianach pliku
	 * wskazanego przez <code>path</code> o typach wskazanych przez
	 * <code>changeTypes</code>. Zmienna <code>changeTypes</code> jest
	 * tablicą zmiennej długości typu integer, wartości dopuszczalne to
	 * statyczne pola klasy <i>FsChangeDescriptor</i>. Użycie innych wartości
	 * całkowitoliczbowych może spowodować nieprzewidywalne zachowanie.
	 * 
	 * @param lfsl
	 *            obiekt implementujący interfejs LocalFsChangeListener
	 * @param path
	 *            sciezka do monitorowanego pliku
	 * @param changeTypes
	 *            typy śledzonych zmian
	 */
	public void registerLocalListener(LocalFsChangeListener lfsl, String path,
			int... changeTypes) {
		synchronized (registeredListings) {
			synchronized (registeredListeners) {
				RegisteredListenersListElem newElem = new RegisteredListenersListElem(
						lfsl, changeTypes);
				ArrayList<RegisteredListenersListElem> tempList;
				if ((tempList = registeredListings.get(path)) != null) {
					int index;

					if ((index = tempList.indexOf(newElem)) >= 0) {
						tempList.get(index).addNewTypes(changeTypes);
					} else {
						tempList.add(newElem);
					}
				} else {
					tempList = new ArrayList<RegisteredListenersListElem>();
					tempList.add(newElem);
					registeredListings.put(path, tempList);

				}
			}
			ArrayList<String> tempPathList;
			if ((tempPathList = registeredListeners.get(lfsl)) != null) {
				if (tempPathList.indexOf(path) == -1) {
					tempPathList.add(path);
				} else
					;
			} else {
				tempPathList = new ArrayList<String>();
				tempPathList.add(path);
				registeredListeners.put(lfsl, tempPathList);
			}
		}
	}

	/**
	 * Usuwanie odbiorcy z listy obslugiwanych obiektów
	 * 
	 * @param lfscl
	 *            Odbiorca, który ma zostać usunięty
	 */

	public void unregisterLocalListener(LocalFsChangeListener lfscl) {
		Object[] pathTable = new Object[0];
		synchronized (registeredListeners) {
			//TODO null pointer exception jak sie 
			//probuje wtyrejestrowac nieisrniejacy
			pathTable = registeredListeners.get(lfscl).toArray();
			if (pathTable ==null)
				return;
		}
		for (Object s : pathTable) {
			unregisterLocalListener(lfscl, (String)s, 0xFFFFFFFF);
		}
	}

	/**
	 * Wyłączenie nasłuchu danego pliku
	 * 
	 * @param lfscl
	 *            Lokalny odbiorca zdarzen systemu plików
	 * @param path
	 *            Ścieżka na której odbiorca nasłuchuje
	 */

	public void unregisterLocalListener(LocalFsChangeListener lfscl, String path) {
		unregisterLocalListener(lfscl, path, 0xFFFFFFFF);
	}

	/**
	 * Wyłączanie poszczególnych typów nasłuchów.
	 * 
	 * @param lfscl
	 *            Lokalny odbiorca zdarzen systemu plików
	 * @param path
	 *            Ścieżka na której odbiorca nasłuchuje
	 * @param changeTypes
	 *            Typy zmian jakie mają zostać usunięte z listy nasłuchu -
	 *            informajce o tych zdarzeniach nie będą już więcej przekazywane
	 *            do odbiorcy.
	 */

	public void unregisterLocalListener(LocalFsChangeListener lfscl,
			String path, int... changeTypes) {
		synchronized (registeredListings) {
			ArrayList<RegisteredListenersListElem> tempRListenersList;
			if ((tempRListenersList = registeredListings.get(path)) != null) {
				RegisteredListenersListElem tempElem = new RegisteredListenersListElem(
						lfscl, 0);
				if (tempRListenersList.contains(tempElem)) {
					int index = tempRListenersList.indexOf(tempElem);
					if (tempRListenersList.get(index).constraintListening(
							changeTypes) == 0) {
						tempRListenersList.remove(index);
						if (tempRListenersList.size() == 0) {
							registeredListings.remove(path);
						}
						synchronized (registeredListeners) {
							registeredListeners.get(lfscl).remove(path);
							if (registeredListeners.get(lfscl).size() == 0) {
								registeredListeners.remove(lfscl);
							}
						}
					}
				}
			}

		}

	}

}
