package twcsckernel.clientKernel.utils;

import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.IOException;
import java.rmi.RemoteException;
import java.text.ParseException;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.text.MaskFormatter;

import twcsckernel.clientKernel.utils.fileChooserClasses.FileSystemModel;
import twcsckernel.clientKernel.utils.fileChooserClasses.FileSystemUtils;
import twcsckernel.projectbase.io.FileDescriptor;
import twcsckernel.projectbase.io.RemoteFile;

/**
 * 
 * Narzędzie do graficznego wyboru plików zdalnych. Po utworzeniu nowej
 * instancji wywolujemy metode <code>showDialog()</code>
 * 
 * @author VMDGroup
 */
public class RemoteFileChooser extends JPanel implements MouseListener {
	/**
	 * 
	 */
	public static final int OK_OPTION = JOptionPane.OK_OPTION;

	public static final int CANCEL_OPTION = JOptionPane.CANCEL_OPTION;

	private static final long serialVersionUID = 1L;

	private String filter;

	private String filename = "";

	private FileSystemModel fsm;

	private final JTextField jtf = new JTextField();

	private JTable fileTable;

	private Vector<String> filterNames = new Vector<String>();

	private Vector<String> filterRegExp = new Vector<String>();

	private JComboBox jcb;

	/**
	 * Otwiera okno dialogowe wyboru pliku. Po wybraniu pliku jego sciezke
	 * pobiera się za pomocą metody <i>getFilename().</i> Możliwe jest
	 * ustawienie filtru za pomocą metody <i>setFilter(String)</i>. Możliwa
	 * jest niespójność między wyświetlaną listą plików a stanem systemu plików
	 * na serwerze, jeśli nastąpiła jego zmiana w czasie kiedy bieżący katalog
	 * był wyświetlony. Możliwe jest więc wystąpienie błędu otwarcia zdalnego
	 * pliku.
	 * 
	 * @return Wartość zwracana : <code> RemoteFileChooser.OK_OPTION</code>,<code>RemoteFileChooser.CANCEL_OPTION</code>
	 */
	private int showDialog() {
		filename = null;
		fsm = new FileSystemModel("/", filterRegExp
				.get(filterRegExp.size() - 1));
		fileTable = new JTable(fsm);
		Container zawartosc = Box.createVerticalBox();
		fileTable.addMouseListener(this);
		JScrollPane jsp = new JScrollPane(fileTable);
		jsp.setSize(200, 200);
		zawartosc.add(jsp);
		zawartosc.add(jtf);
		jcb = new JComboBox(filterNames);
		jcb.setSelectedIndex(filterNames.size() - 1);
		jcb.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fsm.setFilterRegExp(filterRegExp.get(jcb.getSelectedIndex()));
				fsm.filter();
			}

		});
		zawartosc.add(jcb);
		this.add(zawartosc);
		int wynik = OK_OPTION;
		while (wynik == OK_OPTION && filename == null) {
			wynik = JOptionPane.showOptionDialog(null, this, "Wybierz plik",
					JOptionPane.OK_CANCEL_OPTION,
					JOptionPane.INFORMATION_MESSAGE, null, null, null);
			if (!jtf.getText().equals("")) {
				filename = FileSystemUtils.preparePath(fsm.getCurrentDir()
						.concat("/" + jtf.getText()));
			}
		}
		return wynik;
	}

	public int showOpenDialog() {
		jtf.setEditable(false);
		return showDialog();
	}

	public int showSaveDialog() {
		jtf.setEditable(true);
		return showDialog();
	}

	public RemoteFileChooser() {
		filterNames.add("Wszystkie pliki");
		filterRegExp.add(".*");

	}

	/**
	 * 
	 * @return Ciąg znaków okreslający bieżący filtr.
	 */
	public String getFilter() {
		return filter;
	}

	/**
	 * Ustawianie filtru dla plików wyświetlanych. Domyślnie zawinstalowany jest
	 * filtr wyświetlający wszystkie pliki.
	 * 
	 * @param filtername -
	 *            określa nazwę dla filtru, np "Dokumenty word"
	 * @param filterRegExpr -
	 *            określa wyrażenie regularne określające wygląd nazwy pliku, np
	 *            <code>".*\\.(?i)doc"
	 */

	public void addFilter(String filtername, String filterRegExpr) {
		this.filterNames.add(filtername);
		this.filterRegExp.add(filterRegExpr);

	}

	/**
	 * 
	 * @return Pełna ścieżka do pliku zdalnego.
	 * 
	 */

	public String getFilename() {
		return filename;
	}

	public void mouseClicked(MouseEvent e) {
		if (e.getButton() == MouseEvent.BUTTON1) {
			FileDescriptor fd = fsm.getSelectedFiles(fileTable.rowAtPoint(e
					.getPoint()));
			if (!fd.isFile()) {
				jtf.setText("");
				String tmpstr = fsm.getCurrentDir().concat("/" + fd.getName());
				fsm.setCurrentDir(tmpstr);
				fsm.list();
				fsm.filter();
			} else {
				jtf.setText(fd.getName());
			}
		} else if (e.getButton() == MouseEvent.BUTTON3) {

			JPopupMenu jpm = new JPopupMenu();
			JMenuItem item = new JMenuItem("Stwórz katalog");
			item.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					RemoteFile newDir = null;
					String answer = JOptionPane.showInputDialog(fileTable,
							"Podaj nazwę katalogu", "Nowy katalog...",
							JOptionPane.INFORMATION_MESSAGE);
					if (answer != null && answer.length() != 0) {
						try {
							newDir = CommonVariablesContainer.rff
									.newRemoteFile(fsm.getCurrentDir() + answer);
						} catch (RemoteException e1) {
							return;
						} catch (SecurityException e1) {
							return;
						} catch (IOException e1) {
							return;
						}
						if (newDir.mkdir()) {
							fsm.list();
							fsm.filter();
						}
						newDir.disposeFileHandle();
					} else
						return;

				}
			});
			jpm.add(item);
			jpm.show(fileTable, e.getX(), e.getY());
		}

	}

	public void mousePressed(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseReleased(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseEntered(MouseEvent e) {
		// TODO Auto-generated method stub

	}

	public void mouseExited(MouseEvent e) {
		// TODO Auto-generated method stub

	}

}