/*
 * FileSystemModel.java
 *
 * Created on 27 styczeń 2006, 21:48
 */

package twcsckernel.clientKernel.utils.fileChooserClasses;

import java.io.IOException;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.regex.Pattern;

import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableModel;

import twcsckernel.clientKernel.utils.CommonVariablesContainer;
import twcsckernel.projectbase.io.FileDescriptor;
import twcsckernel.projectbase.io.ItemNotFoundException;
import twcsckernel.projectbase.io.RemoteFile;

/**
 * Implementation of a {@link TableModel} representing a filesystem. The table
 * model should contain at least the following columns:
 * <ol>
 * <li>file name</li>
 * <li>file extension (or &lt;dir&gt; for folders)</li>
 * <li>file size (or empty for folders)</li>
 * </ol>
 * @author VMD Group
 */
// implements TableModel
public final class FileSystemModel extends AbstractTableModel {

    private static final long serialVersionUID = 1369681020825310841L;

    /**
     * Currently modelled folder.
     */
    private String filterRegExp;
    
    private String currentDir;

    private FileDescriptor[] filelist;

    private ArrayList<FileDescriptor>filteredFileList = new ArrayList<FileDescriptor>(); 
    
    private static final String[] columnName = new String[] { "name", "ext.",
            "size" };

    //private final List listeners = new ArrayList();
    
    public FileSystemModel(String currentDir, String regExp) {
    	filterRegExp = regExp;
    	this.currentDir=FileSystemUtils.preparePath(currentDir);
    	list();
    	filter();
    }

    public int getRowCount() {
        return filteredFileList.size();
    }

    public int getColumnCount() {
        return 3;
    }

    public String getColumnName(int columnIndex) {
        return columnName[columnIndex];
    }

    public Class<?> getColumnClass(int columnIndex) { // Class<?>
        // What are the types of values returned for each column?
        return String.class;
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
     
    	String value = new String();
        switch (columnIndex) {
        case 0:
        	if (!filteredFileList.get(rowIndex).isFile()) {
                    value = filteredFileList.get(rowIndex).getName();
                } else {
                	value = FileSystemUtils.fileNameWhitoutExt(filteredFileList.get(rowIndex).getName());
                }
            break;

        case 1:
                if (!filteredFileList.get(rowIndex).isFile()) {
                       	value = "<dir>";
                } else {
                    value = FileSystemUtils.fileNameExt(filteredFileList.get(rowIndex).getName());
                }
            
            break;

        case 2:
                if (!filteredFileList.get(rowIndex).isFile()) {
                    value = "0";
                } else {
                    value = Long.toString(filteredFileList.get(rowIndex).getLength());
                }
            }

        return value;
   }

       
    public FileDescriptor getSelectedFiles(int selectedRow) {
        return filteredFileList.get(selectedRow);
         
        
        
    }
    
    public String getCurrentDir() {
        return currentDir;
    }
    
    public void setCurrentDir(String curDir) {
        currentDir = FileSystemUtils.preparePath(curDir);
    }
    public void list(){
    	FileDescriptor tempList[];
        int destIndex=0;
        try {
        	RemoteFile remoteFile = CommonVariablesContainer.rff.newRemoteFile(currentDir); 
    		tempList = remoteFile.list();
			remoteFile.disposeFileHandle();
    		if (currentDir.equals("/")){
				filelist = new FileDescriptor[tempList.length];
			}else{
				filelist = new FileDescriptor[tempList.length+1];
				filelist[0]=new FileDescriptor("..", currentDir.concat("/.."), 0, false, false, 0);
				destIndex++;
			}
			System.arraycopy(tempList, 0, filelist, destIndex, tempList.length);
			
    	} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ItemNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
    }

	public String getFilterRegExp() {
		return filterRegExp;
	}

	public void setFilterRegExp(String filterRegExp) {
		this.filterRegExp = filterRegExp;
	}
	
	public void filter(){
		filteredFileList.clear();
		for (FileDescriptor fd : filelist){
			if (fd.isFile() && Pattern.matches(filterRegExp, fd.getName())){
				filteredFileList.add(fd);
			}
			else if (!fd.isFile()){
				filteredFileList.add(fd);
			}
		}
	
		fireTableDataChanged();
	}


}
