package twcsckernel.serverKernel.io;

import twcsckernel.projectbase.common.FsChangeListener;
import twcsckernel.projectbase.io.LocalFsChangeListener;
import twcsckernel.projectbase.io.UserRootPathManager;
import twcsckernel.serverKernel.impl.UserImpl;

/**
 * Interfejs wpółdzielonego listenera zmian w systemie plików. Przechowuje
 * listenery zdalne zgłoszone przez klientów zdalnych. Jest on przekazywany do
 * fabryk plików poszczególnych użytkowników (nie zaś same listenery zdalne) i
 * pełni rolę pośrednika (proxy), przez co można realizować w różny sposób
 * politykę wymiany zdarzeń pomiędzy różnymi użytkownikami. Fakt, że dziedziczy
 * implementację <i>Remote</i> jest bez znaczenia.
 * 
 * @author VMD Group
 * 
 */
public interface SharedFsChangeListener extends LocalFsChangeListener {

	/**
	 * Metoda pozwala na dodanie listenera, który należy do podanego
	 * użytkownika. Parametr <i>rootManager</i> jest managerem ścieżki root
	 * zastosowanym dla zdarzen tego listenera. Przekazywane zdarzenia posiadają
	 * globalne ścieżki do plików i tłumaczenie na ścieżki względne użytkownika
	 * leży w kompetencji tej klasy właśnie przy pomocy przekazanego root
	 * managera.
	 * 
	 * @param fsChL -
	 *            dodawany listener
	 * @param owner -
	 *            właściciel listenera
	 * @param rootManager -
	 *            manager ścieżki root zastosowany dla zdarzenia
	 */
	public void addFsChangeListener(FsChangeListener fsChL, UserImpl owner,
			UserRootPathManager rootManager);

	/**
	 * Metoda pozwala na ręczne usunięcie podanego listenera.
	 * 
	 * @param fsChL -
	 *            listener do usunięcia
	 * @param owner -
	 *            właściciel listenera
	 * @return - <code>true</code> jeśli listener był zarejestrowany i został
	 *         usunięty lub <code>false</code> w przeciwnym wypadku
	 */
	public boolean removeFsChangeListener(FsChangeListener fsChL, UserImpl owner);

	/**
	 * Metoda pozwala usunąć listenery zdalne danego użytkownika (potrzebne np.
	 * przy wologowaniu użytkownika).
	 * 
	 * @param owner -
	 *            użytkownik
	 */
	public void removeUserListeners(UserImpl owner);

}
