package twcsckernel.serverKernel.io;

import java.io.IOException;
import java.util.Map;

import twcsckernel.serverKernel.utils.TypeData;

/**
 * Interfejs operacji I/O dla zarejestrowanych typów użytkowników.
 * 
 * @author VMD Group
 * 
 */
public interface TypeIO {
	/**
	 * Wczytuje zarejestrowane typy użytkowników.
	 * 
	 * @return - mapa zarejestrowanych typów <nazwa_typu,dane>
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu odczytu
	 */
	public Map<String, TypeData> readTypes() throws IOException;

	/**
	 * Metoda dodaje dane do zbioru zarejestrowanych typów.
	 * 
	 * @param typeData -
	 *            dane do dodania
	 * @return - <code>true</code> jeśli operacja się udała (nie było
	 *         zarejestrowanego typu o podanej nazwie) lub <code>false</code>
	 *         w przeciwnym wypadku.
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu zapisu
	 */
	public boolean addNewType(TypeData typeData) throws IOException;

	/**
	 * Metoda usuwa dane ze zbioru zarejestrowanych typów.
	 * 
	 * @param name -
	 *            nazwa typu do usunięcia
	 * @param types -
	 *            aktualna mapa typów (z której również zostanie typ usunięty)
	 * @return - <code>true</code> jeśli operacja się udała (był
	 *         zarejestrowany typ o podanej nazwie) lub <code>false</code> w
	 *         przeciwnym wypadku.
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu zapisu
	 */
	public boolean removeType(String name, Map<String, TypeData> types)
			throws IOException;

}
