package twcsckernel.serverKernel.io;

import java.io.IOException;
import java.util.Map;

import twcsckernel.serverKernel.utils.UserData;

/**
 * Interfejs operacji I/O dla zarejestrowanych użytkowników.
 * 
 * @author VMD Group
 * 
 */
public interface UserIO {

	/**
	 * Wczytuje zarejestrowanych użytkowników.
	 * 
	 * @return - mapa zarejestrowanych użytkowników <login,dane>
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu odczytu
	 */
	public Map<String, UserData> readUsers() throws IOException;

	/**
	 * Metoda dodaje dane do zbioru zarejestrowanych użytkowników.
	 * 
	 * @param userData -
	 *            dane do dodania
	 * @return - <code>true</code> jeśli operacja się udała (nie było
	 *         zarejestrowanego użytkownika o podanym loginie) lub
	 *         <code>false</code> w przeciwnym wypadku.
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu zapisu
	 */
	public boolean addNewUser(UserData userData) throws IOException;

	/**
	 * Metoda usuwa dane ze zbioru zarejestrowanych użytkowników.
	 * 
	 * @param login -
	 *            nazwa użytkownika do usunięcia
	 * @param users -
	 *            aktualna mapa zarejestrowanych użytkowników (z której również
	 *            zostanie użytkownik usunięty)
	 * @return - <code>true</code> jeśli operacja się udała (był
	 *         zarejestrowany użytkownik o podanym loginie) lub
	 *         <code>false</code> w przeciwnym wypadku.
	 * @throws IOException -
	 *             wyjątek rzucany w przypadku błędu zapisu
	 */
	public boolean removeUser(String login, Map<String, UserData> users)
			throws IOException;

}
