package twcsckernel.serverKernel.plugins;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * Uchwyt plugina na serwerze zawierający komplet informacji tj.: deskryptor,
 * ścieżki do plików JAR klienta i serwera oraz instancję PluginBuildera
 * (o ile plugina pracuje w trybie zdalnymm czyli deskryptor.isRemote==true).
 * 
 * @author VMD Group
 * 
 */
public class PluginHandle {

	/**
	 * Deskryptor plugina.
	 */
	public final PluginDescriptor descriptor;

	private final String pluginPath;

	private final String serverFilename;

	private final String clientFilename;
	
	private final byte[] clientPluginData;

	/**
	 * klasa serwerowej części plugina - jeśli plugin nie jest przeznaczony do
	 * pracy zdalnej, to wartością jest <code>null</code>.
	 */
	public final PluginBuilder serverPluginBuilder;

	/**
	 * @return ścieżka do JARa serwerowej części plugina - jeśli plugin nie
	 *         posiada części serwerowej, to wartością jest <code>null</code>.
	 */
	public String getServerJarPath() {
		if (descriptor.isRemote)
			return pluginPath + "/" + serverFilename;
		else
			return null;
	}

	/**
	 * @return ścieżka do JARa klienckiej części plugina.
	 */
	public String getClientJarPath() {
		return pluginPath + "/" + clientFilename;
	}
	
	/**
	 * Metoda zwraca binarną postać pliku JAR klienckiej części plugina.
	 * @return - binarna postać pliku JAR klienta
	 */
	public byte[] getClientPluginData() {
		/*
		 * Można dla bezpieczeństwa zrobić:
		 * return clientPluginData.clone();
		 * ale to mogłoby być zbyt pamięciożerne.
		 * */
		return clientPluginData;
	}

	public PluginHandle(PluginBuilder serverPluginBuilder,
			PluginDescriptor descriptor, String pluginPath,
			String serverFilename, String clientFilename, byte[] clientPluginData) {
		this.clientFilename = clientFilename;
		this.descriptor = descriptor;
		this.pluginPath = pluginPath;
		this.serverFilename = serverFilename;
		this.serverPluginBuilder = serverPluginBuilder;
		this.clientPluginData=clientPluginData;
	}

}
