﻿package twcsckernel.serverKernel.plugins;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * PluginContainer jest kontenerem klas pluginów dostępnych dla danego typu
 * użytkownika.
 * 
 * @author VMD Group
 * 
 */
public class PluginHandleContainer {
	Map<String, Set<PluginHandle>> pluginTypeMap;

	Map<String, PluginHandle> clientPluginPkgMap;

	public PluginHandleContainer() {
		pluginTypeMap = new HashMap<String, Set<PluginHandle>>();
		clientPluginPkgMap = new HashMap<String, PluginHandle>();
	}

	/**
	 * Usuwa typ użytkownika.
	 * 
	 * @param userType
	 *            typ użytkownika
	 * @return true gdy usunięto, false gdy nie znaleziono takiej nazwy typu
	 */
	public boolean removeUserType(String userType) {
		if (pluginTypeMap.get(userType) == null)
			return false;
		pluginTypeMap.remove(userType);
		return true;
	}

	/**
	 * Dodaje klasę pluginu do listy dostępnych dla danego typu użytkownika.
	 * 
	 * @param userType -
	 *            typ użytkoniwka
	 * @param plugin -
	 *            klasa pluginu
	 * @return - <i>true</i> jeśli udało się dodać plugin lub
	 *         <code>false</code> w przeciwnym wypadku
	 */
	public boolean addPluginToUserType(String userType, PluginHandle plugin) {
		Set<PluginHandle> userSet = pluginTypeMap.get(userType);
		if (userSet == null) {
			userSet = new HashSet<PluginHandle>();
			pluginTypeMap.put(userType, userSet);
		}

		if (clientPluginPkgMap.get(plugin.descriptor.clientPackagePath) == null)
			clientPluginPkgMap.put(plugin.descriptor.clientPackagePath, plugin);

		return userSet.add(plugin);
	}

	/**
	 * Zwraca tablicę uchwytów pluginów dostępnych dla danego typu użytkownika.
	 * 
	 * @param userType -
	 *            nazwa typu użytkownika
	 * @return talica klas pluginów (lub <code>null</code> jeśli nie ma
	 *         takiego typu użytkownika)
	 */
	public PluginHandle[] getUserTypePlugins(String userType) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		if (userPluginSet == null)
			return null;
		return userPluginSet.toArray(new PluginHandle[0]);
	}

	/**
	 * Metoda zwraca tablicę deskryptorów pluginów dla danego typu użytkownika
	 * 
	 * @param userType -
	 *            typ użytkownika
	 * @return - talibca deskryptorów pluginów lub <code>null</code> jeśli
	 *         podany typ nie istnieje
	 */
	public PluginDescriptor[] getUserPluginsDescriptors(String userType) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		if (userPluginSet == null)
			return new PluginDescriptor[0];
		PluginDescriptor[] descriptors = new PluginDescriptor[userPluginSet
				.size()];
		int index = 0;
		for (PluginHandle pluginHandle : userPluginSet) {
			descriptors[index] = pluginHandle.descriptor;
			index++;
		}
		return descriptors;
	}

	/**
	 * Metoda zwraca uchwyt plugina danego typu użytkownika o wyspecyfikowanej
	 * ścieżce pakietowej części klienckiej.
	 * 
	 * @param userType -
	 *            typ użytkownika
	 * @param clientPackagePath -
	 *            ścieżka pakietowa klienckiej części plugina
	 * @return - uchwyt szukanego plugina lub <code>null</code> jeśli nie
	 *         istnieje typ użytkownika lub plugin o podanej ścieżce
	 */
	public PluginHandle getUserPluginByClientPackagePath(String userType,
			String clientPackagePath) {
		Set<PluginHandle> userPluginSet = pluginTypeMap.get(userType);
		PluginHandle selectedPlugin = clientPluginPkgMap.get(clientPackagePath);
		if (userPluginSet.contains(selectedPlugin))
			return selectedPlugin;
		return null;

	}
}
