package twcsckernel.serverKernel.usr;

import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import twcsckernel.serverKernel.impl.UserImpl;

/**
 * Klasa odpowiedzialna za wylogowanie użytkowników, którzy nie wykazywali
 * aktywności dłużej niż przez zadany czas. Pełni funkcję user garbage
 * collectora.
 * 
 * @author VMD Group
 * 
 */
public class UserGarbageCollector {

	private long interval;

	private Timer timer = null;

	private Object timerMutex = new Object();

	private Set<UserImpl> registeredUserSet = null;

	private TimerTask gcTask = new TimerTask() {

		@Override
		public void run() {
			synchronized (timerMutex) {
				if (timer == null)
					return;
				synchronized (registeredUserSet) {
					long newStamp = System.currentTimeMillis();
					//wykonywana jest kopia ze wzgledu na to, ze
					//user.kickUser() modyfikuje posrednio
					//registeredUserSet
					UserImpl[] tmpUsrCopy = registeredUserSet
							.toArray(new UserImpl[0]);
					long diff;
					for (UserImpl user : tmpUsrCopy) {
						diff = newStamp - user.getTimeStamp();
						if (diff > interval)
							user.kickUser();
					}
				}
			}

		}
	};

	/**
	 * Metoda ustawia zbiór zarejestrowanych użytkowników na którym będzie
	 * działać user garbage collector.
	 * 
	 * @param registeredUserSet
	 *            zbiór zarejestrowanych użytkowników
	 */
	public void setUserSet(Set<UserImpl> registeredUserSet) {
		synchronized (timerMutex) {
			boolean ifEnabled = timer != null;
			disable();
			this.registeredUserSet = registeredUserSet;
			if (ifEnabled)
				enable();
		}

	}

	/**
	 * Metoda wyłącza działanie garbage collectora.
	 */
	public void disable() {
		synchronized (timerMutex) {
			if (timer != null) {
				timer.cancel();
				timer = null;
			}
		}
	}

	/**
	 * Metoda włącza działanie garbage collectora o ile zbiór użytkowników
	 * został ustawiony oraz zadany interwał czasowy jest większy od zera.
	 */
	public void enable() {
		synchronized (timerMutex) {
			if (registeredUserSet == null)
				return;
			if (interval == 0) {
				disable();
				return;
			}
			timer = new Timer(true);
			timer.schedule(gcTask, interval, interval);
		}
	}

	/**
	 * Metoda ustawia interwał czasowy co który user garbage collector będzie
	 * przeglądał stan użytkowników.
	 * 
	 * @param interval -
	 *            interwał czasowy w milisekundach
	 */
	public void setInterval(long interval) {
		synchronized (timerMutex) {
			boolean ifEnabled = timer != null;
			disable();
			this.interval = interval;
			if (ifEnabled)
				enable();
		}
	}

}
