package twcsckernel.serverKernel.utils;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;
import java.util.Set;

import twcsckernel.serverKernel.io.TypeIO;

/**
 * Kontener zarejestrowanych typów użytkowników wraz z zarządzaniem.
 * 
 * @author VMD Group
 * 
 */
public class RegisteredTypes {
	private Map<String, TypeData> types = null;

	TypeIO typeIO;

	/**
	 * Konstruktor ustawia podany manager I/O i odczytuje wszystkie typy.
	 * 
	 * @param typeIO -
	 *            obiekt zarządzający operacjami I/O na typach
	 * @throws IOException -
	 *             błąd odczytu
	 */
	public RegisteredTypes(TypeIO typeIO) throws IOException {
		this.typeIO = typeIO;
		types = Collections.synchronizedMap(typeIO.readTypes());
	}

	/**
	 * Metoda dodaje typ użytkownika do zbioru danych zarządzanego przez podany
	 * w konstruktorze obiekt I/O
	 * 
	 * @param data -
	 *            dane typu do dodania
	 * @return - <code>true</code> jeśli operacja się powiodła (nie było
	 *         wcześniej typu o podanej nazwie) lub <code>false</code> w
	 *         przeciwnym wypadku
	 * @throws IOException -
	 *             błąd zapisu
	 */
	public boolean addNewType(TypeData data) throws IOException {
		if (!types.containsKey(data.getTypeName()))
			return typeIO.addNewType(data);
		return false;
	}

	/**
	 * Metoda usuwa typ użytkownika ze zbioru danych zarządzanego przez podany w
	 * konstruktorze obiekt I/O
	 * 
	 * @param name -
	 *            nazwa typu do usunięcia
	 * @return - <code>true</code> jeśli operacja się powiodła lub
	 *         <code>false</code> w przeciwnym wypadku
	 * @throws IOException -
	 *             błąd zapisu
	 */
	public boolean removeType(String name) throws IOException {
		return typeIO.removeType(name, types);
	}

	/**
	 * @return - tablica wszystkich typów
	 */
	public TypeData[] getRegisteredTypes() {
		return types.values().toArray(new TypeData[0]);
	}

	/**
	 * @return - zbiór wszystkich nazw typów
	 */
	public Set<String> getTypeNamesSet() {
		return types.keySet();
	}

}
