package twcsckernel.serverKernel.utils;

import java.io.IOException;
import java.util.Collections;
import java.util.Map;

import twcsckernel.serverKernel.io.UserIO;

/**
 * Kontener zarejestrowanych użytkowników wraz z zarządzaniem.
 * 
 * @author VMD Group
 * 
 */
public class RegisteredUsers {
	Map<String, UserData> logins = null;

	UserIO userIO;

	/**
	 * Konstruktor ustawia podany manager I/O i odczytuje wszystkich
	 * użytkowników.
	 * 
	 * @param userIO -
	 *            obiekt zarządzający operacjami I/O użytkowników
	 * @throws IOException -
	 *             błąd odczytu
	 */
	public RegisteredUsers(UserIO userIO) throws IOException {
		this.userIO = userIO;
		logins = Collections.synchronizedMap(userIO.readUsers());
	}

	/**
	 * Metoda sprawdza czy podane hasło w postaci jawnej zgadza się z
	 * zahaszowanym hasłem z bazy użytkowników.
	 * 
	 * @param login -
	 *            login użytkownika
	 * @param passwd -
	 *            hasło użytkownika
	 * @return - <code>true</code> jeśli hasło zostało poprawnie zweryfikowane
	 *         lub <code>false</code> w przeciwnym wypadku
	 */
	public boolean checkPassword(String login, String passwd) {
		if (logins.get(login) != null) {
			UserData userData = logins.get(login);
			if (Encoder.hashPassword(passwd)
					.equals(userData.getEncodedPasswd()))
				return true;
		}
		return false;

	}

	/**
	 * Metoda dodaje nowego użytkownika do bazy zarejestrowanych użytkowników.
	 * 
	 * @param data -
	 *            dane użytkownika do dodania
	 * @return - <code>true</code> jeśli operacja się powiodła (nie było
	 *         takiego użytkonika) lub <code>false</code> w przeciwnym wypadku
	 * @throws IOException -
	 *             błąd zapisu
	 */
	public boolean addNewUser(UserData data) throws IOException {
		if (!logins.containsKey(data.getLogin()))
			return userIO.addNewUser(data);
		return false;
	}

	/**
	 * Metoda usuwa użytkownika o podanym loginie z bazy.
	 * 
	 * @param login -
	 *            login użytkownika do usunięcia
	 * @return - <code>true</code> jeśli operacja się powiodła lub
	 *         <code>false</code> w przeciwnym wypadku
	 * @throws IOException - błąd zapisu
	 */
	public boolean removeUser(String login) throws IOException {
		return userIO.removeUser(login, logins);
	}

	/**
	 * @return - talibca zarejestrowanych użytkowników
	 */
	public UserData[] getRegisteredUsers() {
		return logins.values().toArray(new UserData[0]);
	}

	/**
	 * @param login - login użytkownika
	 * @return - dane użytkownika o wskazanym loginie
	 */
	public UserData getUserData(String login) {
		return logins.get(login);
	}

}
