package twcsckernel.serverKernel.utils;

import twcsckernel.projectbase.io.FileDescriptor;
import twcsckernel.projectbase.io.FsChangeDescriptor;
import twcsckernel.projectbase.io.UserRootPathManager;

/**
 * Klasa służy dostosowywaniu zdarzeń o zmianach w systemie plików względem
 * managera ścieżki root danego użytkownika.
 * 
 * @author VMD Group
 * 
 */
public class RootAdjuster {
	/**
	 * Dostosowanie ścieżek dla deskryptora zmian
	 * 
	 * @param fsDescriptor -
	 *            deskryptor zmian
	 * @param rootManager -
	 *            manager ścieżki root używany do "obróbki"
	 * @return - dostosowany przez rootManagera deskryptor zmiany
	 */
	public static FsChangeDescriptor adjustRootPath(
			FsChangeDescriptor fsDescriptor, UserRootPathManager rootManager) {
		FsChangeDescriptor result = new FsChangeDescriptor(fsDescriptor
				.getChangeType());
		if (fsDescriptor.getSource() != null) {
			String tmpStr = adjustRootPath(fsDescriptor.getSource(),
					rootManager);
			if (tmpStr == null)
				return null;
			result.setSource(tmpStr);
		}
		if (fsDescriptor.getTarget() != null) {
			String tmpStr = adjustRootPath(fsDescriptor.getTarget(),
					rootManager);
			if (tmpStr == null)
				return null;
			result.setTarget(tmpStr);
		}
		if (fsDescriptor.getChangeInfo() != null) {
			FileDescriptor tmp = adjustRootPath(fsDescriptor.getChangeInfo(),
					rootManager);
			if (tmp == null)
				return null;
			result.setChangeInfo(tmp);
		}
		return result;
	}

	/**
	 * Dostosowanie ścieżek dla deskryptora pliku
	 * 
	 * @param changeInfo -
	 *            deskryptor pliku
	 * @param rootManager -
	 *            manager ścieżki root używany do "obróbki"
	 * @return - dostosowany przez rootManagera deskryptor pliku
	 */
	public static FileDescriptor adjustRootPath(FileDescriptor changeInfo,
			UserRootPathManager rootManager) {
		String tmp = adjustRootPath(changeInfo.getPath(), rootManager);
		if (tmp == null)
			return null;
		FileDescriptor result = new FileDescriptor();
		result.setPath(tmp);
		result.setHidden(changeInfo.isHidden());
		result.setLastModified(changeInfo.getLastModified());
		result.setIsFile(changeInfo.isFile());
		result.setLength(changeInfo.getLength());
		result.setName(changeInfo.getName());
		return result;
	}

	/**
	 * Dostosowanie podanej ścieżki absolutnej przez manager ścieżki root
	 * 
	 * @param absolutePath -
	 *            ścieżka absolutna do dostosowania
	 * @param rootManager -
	 *            manager ścieżki root używany do "obróbki"
	 * @return - ścieżka dostosowana przez rootManagera
	 */
	public static String adjustRootPath(String absolutePath,
			UserRootPathManager rootManager) {
		return rootManager.getUserAbsolutePath(absolutePath);
	}
}
