/**
 * 
 */
package twcsckernel.projectbase.common;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import twcsckernel.projectbase.io.RemoteFile;
import twcsckernel.projectbase.io.RemoteFileInputStream;
import twcsckernel.projectbase.io.RemoteFileOutputStream;

public interface RemoteFileSystem extends Remote {

	/**
	 * Tworzy nowy obiekt pliku zdalnego na serwerze o podanej ścieżce.
	 * 
	 * @param path
	 *            ścieżka pliku zdalnego na serwerze (licząc od katalogu root
	 *            użytkownika)
	 * @return obiekt pliku zdalnego
	 * @throws RemoteException
	 * @throws IOException -
	 *             wyjątek jest rzucany jeśli serwer nie może utworzyć ścieżki
	 *             kanonicznej dla podanej ścieżki
	 * @throws SecurityException -
	 *             próba dostępu do zabronionego lub nieprawidłowego pliku
	 */
	public RemoteFile newRemoteFile(String path) throws RemoteException,
			SecurityException, IOException;

	/**
	 * Metoda tworzy nowy zdalny strumień wyjścia.
	 * 
	 * @param filePath -
	 *            ścieżka pliku na serwerze sprawdzana przez odpowiedni manager
	 *            bezpieczeństwa użytkownika
	 * @return - zdalny strumień wyjścia
	 * @throws RemoteException -
	 *             standardowy wyjątek zdalny
	 * @throws FileNotFoundException -
	 *             nie odnaleziono pliku (podana ściezka jest katalogiem)
	 * @throws SecurityException -
	 *             próba dostępu do zabronionego lub nieprawidłowego pliku
	 */
	public RemoteFileOutputStream newFileOutputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException;

	/**
	 * Metoda tworzy nowy zdalny strumień wejścia.
	 * 
	 * @param filePath -
	 *            ścieżka pliku na serwerze sprawdzana przez odpowiedni manager
	 *            bezpieczeństwa użytkownika
	 * @return - zdalny strumień wejścia
	 * @throws RemoteException -
	 *             standardowy wyjątek zdalny
	 * @throws FileNotFoundException -
	 *             nie odnaleziono pliku
	 * @throws SecurityException -
	 *             próba dostępu do zabronionego lub nieprawidłowego
	 *             pliku/katalogu
	 */
	public RemoteFileInputStream newFileInputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException;
}
