﻿package twcsckernel.projectbase.common;

import java.io.IOException;
import java.rmi.Remote;
import java.rmi.RemoteException;

import twcsckernel.projectbase.plugins.PluginDescriptor;

/**
 * Interfejs zdalny użytkownika.
 * 
 * @author VMD Group
 * 
 */
public interface User extends Remote {

	/**
	 * Listowanie dostępnych pluginów dla klienta
	 * 
	 * @return tablica deskryptorów dostępnych pluginów
	 * @throws RemoteException
	 */
	public PluginDescriptor[] listClientPlugins() throws RemoteException;

	/**
	 * Download klienckiego JARa plugina podanej nazwie pakietowej.
	 * 
	 * @param clientPackagePath -
	 *            nazwa pakietowa klienckiej części plugina
	 * @return - tablica bajtów pliku JAR lub <code>null</code> jeśli nie
	 *         znaleziono plugina o podanej nazwie
	 * @throws RemoteException
	 * @throws IOException -
	 *             wyjątek rzucany gdy wystąpił wyjątek podczas odczytu plugina
	 *             z dysku
	 */
	public byte[] downloadClientPlugin(String clientPackagePath)
			throws RemoteException, IOException;

	/**
	 * Pobranie agenta sieciowego zdalnej części plugina o podanej klienckiej
	 * ścieżce pakietowej.
	 * 
	 * @param clientPackagePath -
	 *            nazwa pakietowa klienckiej części plugina
	 * @param clientCallback -
	 *            własny agent sieciowy
	 * @return - agent serwerowej części plugina
	 */
	public RemoteAgent getRemotePluginAgent(String clientPackagePath,
			RemoteAgent clientCallback) throws RemoteException;

	/**
	 * Pobiera fabrykę plików danego użytkownika.
	 * 
	 * @return - fabryka plików
	 * @throws RemoteException
	 */
	public RemoteFileSystem getUserFileSystem() throws RemoteException;

	/**
	 * Ustawia na serwerze listener zmian w systemie plików.
	 * 
	 * @param fsChangeListener -
	 *            listener zmian w systemie plików
	 * @throws RemoteException
	 */
	public void setFsChangeListener(FsChangeListener fsChangeListener)
			throws RemoteException;

	/**
	 * Wylogowanie użytkownika z serwera.
	 * 
	 * @throws RemoteException
	 */
	public void logout() throws RemoteException;

	/**
	 * Metoda służy do pingowania serwera w celu utrzymania aktywnego
	 * rejestracji użytkownika w serwerze RMI.
	 * 
	 * @throws RemoteException
	 */
	public void ping() throws RemoteException;

	/**
	 * Metoda zwraca maksymalny czas przez jaki użytkownik może nie wykonywać
	 * żadnej metody na obiekcie <i>User</i>. Po przekroczeniu tego czasu
	 * serwer ma prawo wylogować klienta.
	 * 
	 * @return - maksymalny czas nieaktywności w milisekundach
	 * @throws RemoteException
	 */
	public long getActivityTimeout() throws RemoteException;
}
