﻿package twcsckernel.projectbase.io;

import java.io.File;
import java.io.IOException;

/**
 * Interfejs mededżera bezpieczeństwa plików, który sprawdza czy podana zdalnie
 * ścieżka jest poprawna i czy użytkownik ma do niej prawa.
 * 
 * @author VMD Group
 * 
 */
public interface FileSecurityManager {
	public static final int ACC_OK = 0;

	public static final int ACC_NOT_FOUND = 1;

	public static final int ACC_ILLEGAL = 2;

	/**
	 * Sprawdza, czy użytkownik ma prawa do podanej ścieżki (kanoniczna ścieżka
	 * <i>root path</i> użytkownika zawiera się w kanonicznej ścieżce obiektu
	 * file) oraz jeśli prawa są sprawdza czy podany plik istnieje.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - są prawa oraz plik istnieje
	 *            <li><i>ACC_NOT_FOUND</i> - są prawa, ale plik nie istnieje
	 *            lub niepoprawna ścieżka (w ramach praw)
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna ścieżka
	 *            </ul>
	 */
	public int checkReadAccess(File file);

	/**
	 * Sprawdza, czy użytkownik ma prawa do pliku o podanej ścieżce (kanoniczna
	 * ścieżka <i>root path</i> użytkownika zawiera się w kanonicznej ścieżce
	 * obiektu file) oraz jeśli prawa są sprawdza czy podany plik jest plikiem
	 * poprawnym systemu plików.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - są prawa oraz nazwa pliku jest poprawna
	 *            <li><i>ACC_NOT_FOUND</i> - są prawa, ale nazwa pliku nie
	 *            jest poprawna lub niepoprawna ścieżka (w ramach praw)
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna ścieżka
	 *            </ul>
	 */
	public int checkWriteAccess(File file);

	/**
	 * Sprawdza, czy użytkownik ma prawa do ścieżki podanego pliku (kanoniczna
	 * ścieżka <i>root path</i> użytkownika zawiera się w kanonicznej ścieżce
	 * obiektu file) - używana przez
	 * {@link #checkReadAccess(File) checkReadAccess} oraz
	 * {@link #checkWriteAccess(File) checkWriteAccess}.
	 * 
	 * @param file
	 *            sprawdzany obiekt pliku
	 * @return
	 *            <ul>
	 *            <li><i>ACC_OK</i> - są prawa = root path użytkownika zawiera
	 *            się w filePath
	 *            <li><i>ACC_ILLEGAL</i> - brak praw lub niepoprawna ścieżka
	 *            </ul>
	 */
	public int checkRootPath(File file);

	/**
	 * @return menedżer ścieżki root praw użytkownika
	 */
	public UserRootPathManager getUserRootPathManager();

	/**
	 * Ustawienie ścieżki root security managera
	 * 
	 * @param userRootPath -
	 *            ścieżka root użytkownika
	 * @throws IOException -
	 *             ew. wyjątek rzucany przy pobieraniu ścieżki kanonicznej pliku
	 */
	public void setRootPath(String userRootPath) throws IOException;

}
