﻿package twcsckernel.projectbase.io;

import java.io.Serializable;

/**
 * Deskryptor zmian w systemie plików. Określa typ dokonanej zmiany (dodanie
 * pliku,usunięcie,zmiana nazwy etc.) oraz ścieżkę pliku którego zmiana dotyczy
 * i ew. nową ścieżkę jeśli typ zmiany tego wymaga.
 * 
 * @author VMD Group
 * 
 */
public class FsChangeDescriptor implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * Typ dokonanej zmiany
	 */
	private int changeType;

	/**
	 * Plik/katalog którego zmiany dotyczą lub który jest ścieżką źródłową dla
	 * niektórych zmian.
	 */
	private String source=null;

	/**
	 * Plik/katalog który jest ścieżką docelową dla niektórych zmian.
	 */
	private String target=null;

	/**
	 * Deskryptor zmienionego pliku wymagany dla niektórych zmian.
	 */
	private FileDescriptor changeInfo=null;

	/**
	 * Informuje o potrzebie ponownego wylistowania zawartości ścieżki wskazanej
	 * przez <i>source</i>
	 */
	public static final int PATH_REFRESH = 1;

	/**
	 * Informuje o skopiowaniu pliku/katalogu wskazanego w <i>source</i> do
	 * <i>target</i>.
	 */
	public static final int FILE_COPY = 2;

	/**
	 * Informuje o przeniesieniu pliku/katalogu wskazanego w <i>source</i> do
	 * <i>target</i>.
	 */
	public static final int FILE_MOVE = 4;

	/**
	 * Informuje o usunięciu pliku/katalogu wskazanego przez <i>source</i>.
	 */
	public static final int FILE_DELETE = 8;

	/**
	 * Informuje o zmianie nazwy pliku/katalogu wskazanego przez ścieżkę
	 * <i>source</i> na nazwę wskazaną przez <i>target</i>.
	 */
	public static final int FILE_RENAME = 16;

	/**
	 * Informuje o utworzeniu pliku opisanego w <i>changeInfo</i>.
	 */
	public static final int FILE_CREATE = 32;

	/**
	 * Informuje o potrzebie odswiezenia danych o pliku wskanego w sciezce
	 * <i>changeInfo</i> danymi zawartymi również w <i>changeInfo</i>.
	 */
	public static final int FILE_REFRESH = 64;
	
	/**
	 * Informuje o utworzeniu poddrzewa katalogów wskanego w sciezce
	 * <i>changeInfo</i> danymi zawartymi również w <i>changeInfo</i>
	 * (najgłębszego poziomu).
	 */
	public static final int FILE_MKDIRS = 128;
	
	public FsChangeDescriptor(int changeType) {
		this.changeType=changeType;
	}

	public int getChangeType() {
		return changeType;
	}

	public String getTarget() {
		return target;
	}

	public String getSource() {
		return source;
	}

	public FileDescriptor getChangeInfo() {
		return changeInfo;
	}

	public void setChangeInfo(FileDescriptor changeInfo) {
		this.changeInfo = changeInfo;
	}

	public void setChangeType(int changeType) {
		this.changeType = changeType;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public void setTarget(String target) {
		this.target = target;
	}

}
