package twcsckernel.projectbase.io;

import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;

import twcsckernel.projectbase.common.ReaderFactory;

public class RemoteFileInputStream extends InputStream implements Serializable {

	private static final long serialVersionUID = 1L;

	private final ReaderFactory rf;

	private final int streamID;

	private static final IOException streamNotFound = new IOException(
			"Stream not found");

	public RemoteFileInputStream(int streamID, ReaderFactory rf) {
		this.streamID = streamID;
		this.rf = rf;
	}

	@Override
	public int read() throws IOException {
		int result = 0;
		try {
			result = rf.read(streamID).readResult;
		} catch (ItemNotFoundException snfe) {
			throw streamNotFound;
		}
		return result;
	}

	@Override
	public int read(byte[] b, int off, int len) throws IOException {
		ReadResult result = null;
		if (b == null) {
			throw new NullPointerException();
		} else if ((off < 0) || (off > b.length) || (len < 0)
				|| ((off + len) > b.length) || ((off + len) < 0)) {
			throw new IndexOutOfBoundsException();
		} else if (len == 0) {
			return 0;
		}
		try {
			result = rf.read(streamID, len);
		} catch (ItemNotFoundException snfe) {
			throw streamNotFound;
		}
		//System.arraycopy(result.buffer, 0, b, off, result.readResult);
		
		 for (int i=0;i<result.readResult;i++) 
			 b[off+i]=result.buffer[i];
		 

		return result.readResult;
	}

	@Override
	public int read(byte[] b) throws IOException {
		return read(b, 0, b.length);
	}

	@Override
	public long skip(long off) throws IOException {
		long result = 0;
		try {
			result = rf.skip(streamID, off);
		} catch (ItemNotFoundException snfe) {
			throw streamNotFound;
		}
		return result;
	}

	@Override
	public int available() throws IOException {
		int result = 0;
		try {
			result = rf.available(streamID);
		} catch (ItemNotFoundException snfe) {
			throw streamNotFound;
		}
		return result;
	}

	@Override
	public void close() throws IOException {
		try {
			rf.close(streamID);
		} catch (ItemNotFoundException snfe) {
			throw streamNotFound;
		}
	}
}
