package twcsckernel.projectbase.io;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Set;

import twcsckernel.projectbase.common.RemoteFileSystem;

public class RemoteFileSystemImpl implements RemoteFileSystem {

	private Set<LocalFsChangeListener> fsListeners = new HashSet<LocalFsChangeListener>();
	
	public final ReaderFactoryImpl readerFactory;
	public final WriterFactoryImpl writerFactory;
	public final FileFactoryImpl fileFactory;
	public final FileSecurityManager defaultSecurityManager;
	
	
	public RemoteFileSystemImpl(FileSecurityManager manager) {
		defaultSecurityManager = manager;
		fileFactory = new FileFactoryImpl(defaultSecurityManager,fsListeners);
		writerFactory = new WriterFactoryImpl(defaultSecurityManager,fsListeners);
		readerFactory = new ReaderFactoryImpl(defaultSecurityManager);
	}

	public RemoteFile newRemoteFile(String filePath) throws RemoteException,
			SecurityException, IOException {
		return fileFactory.newRemoteFile(filePath);
	}

	public RemoteFileOutputStream newFileOutputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException {
		return writerFactory.newFileOutputStream(filePath);
	}

	public RemoteFileInputStream newFileInputStream(String filePath)
			throws RemoteException, FileNotFoundException, SecurityException {
		return readerFactory.newFileInputStream(filePath);
	}
	
	public boolean addFilesystemChangeListener(LocalFsChangeListener fsChL) {
		if (fsChL != null)
			return fsListeners.add(fsChL);
		else
			return false;
	}

	public boolean removeFilesystemChangeListener(LocalFsChangeListener fsChL) {
		return fsListeners.remove(fsChL);
	}
	
	public void releaseAllResources() {
		readerFactory.releaseAllResources();
		writerFactory.releaseAllResources();
		fileFactory.releaseAllResources();
	}

}
