/*
 * Copyright 2000 Computer System Services, Inc.
 *
 * Permission to use this software for any purpose is granted provided that
 * this copyright notice is preserved.
 *
 * This software is provided as-is and without warranty as to its
 * fitness for any purpose.  In other words, Computer System Services,
 * Inc. does not guarantee that this software works.  It is provided
 * only in the hope that it may be found useful by someone.
 *
 * Please e-mail tttaylor@cssassociates.com if you find any errors
 * or want to request changes/enhancements.
 */
package twcsckernel.projectbase.rmi.socketfactory;

import java.io.*;
import java.net.*;

/**
 * Provides a signalling channel used to set up callback sockets between the
 * client and server.
 * 
 * @author Tim Taylor -- tttaylor@cssassociates.com
 */
public class SignallingChannel extends Thread {
  private String directAddress;
  private int directPort;

  private String destinationAddress;
  private int destinationPort;

  private Socket socket;
  private DataInputStream in;
  private DataOutputStream out;

  public SignallingChannel(final String directAddress, final int directPort)
      throws IOException {
    this.directAddress = directAddress;
    this.directPort = directPort;
    socket = new Socket(directAddress, directPort);

    in = new DataInputStream(socket.getInputStream());
    out = new DataOutputStream(socket.getOutputStream());

    // Register with server
    out.writeInt(TwoWay.PROTOCOL_MAGIC);
    out.writeInt(TwoWay.REGISTER_CALLBACK_SOCKET_SOURCE);
    out.write(socket.getLocalAddress().getAddress());
    out.flush();

    // Get back server endpoint info
    in.readInt(); /*
                   * int opcode = in.readInt(); - zmienna opcode nie byla
                   * czytana-widocznie jakis smiec do wyciagniecia z kanalu;)
                   * nizej podobnie
                   */
    byte[] address = new byte[4];
    in.read(address, 0, 4);
    destinationAddress = TwoWay.getAddressString(address);
    destinationPort = in.readInt();
  }

  public String getDirectAddress() {
    return directAddress;
  }

  public int getDirectPort() {
    return directPort;
  }

  public String getDestinationAddress() {
    return destinationAddress;
  }

  public int getDestinationPort() {
    return destinationPort;
  }

  public DataOutputStream getOutputStream() {
    return out;
  }

  public void run() {
    try {
      for (;;) {
        in.readInt();// int magic = in.readInt(); //j.w.
        in.readInt();// int opcode = in.readInt(); //j.w.
        int port = in.readInt();

        byte[] localAddress = InetAddress.getLocalHost().getAddress();

        Socket localSocket = null;

        try {
          localSocket = new Socket(TwoWay.getAddressString(localAddress), port);
        } catch (IOException e) {
          // Usually means client terminating while server
          // requesting callback socket.
        }

        Socket remoteSocket = new Socket(directAddress, directPort);
        new SocketAdapter(localSocket, remoteSocket);

        DataOutputStream remoteOut = new DataOutputStream(remoteSocket
            .getOutputStream());

        remoteOut.writeInt(TwoWay.PROTOCOL_MAGIC);
        remoteOut.writeInt(TwoWay.RETURN_CALLBACK_SOCKET);
        remoteOut.write(localAddress);
        remoteOut.writeInt(port);
        remoteOut.flush();
      }
    } catch (EOFException e) {
      // Assume the server disconnected. We could add shutdown
      // signalling to the protocol, but we would still find
      // ourselves here if the server abends.
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}
