/**
 * 
 */
package put.cs.vmd.client;

import java.awt.GridLayout;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.server.RMISocketFactory;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import twcsckernel.clientKernel.desktop.VmdDesktopInterface;
import twcsckernel.clientKernel.desktop.VmdDesktopListener;
import twcsckernel.clientKernel.io.FsChangeListenerServer;
import twcsckernel.clientKernel.plugins.ClientPlugin;
import twcsckernel.clientKernel.plugins.ClientPluginLoader;
import twcsckernel.clientKernel.utils.CommonVariablesContainer;
import twcsckernel.clientKernel.utils.ConnectionSustainer;
import twcsckernel.projectbase.common.RemoteAgent;
import twcsckernel.projectbase.common.RemoteServer;
import twcsckernel.projectbase.common.User;
import twcsckernel.projectbase.plugins.PluginDescriptor;
import twcsckernel.projectbase.rmi.registry.LocateRegistry;
import twcsckernel.projectbase.rmi.registry.Registry;
import twcsckernel.projectbase.rmi.socketfactory.ClientTwoWaySocketFactory;
import twcsckernel.projectbase.utils.ClassPathLoader;

/**
 * 
 */
public class VMDClient {

    private static final String pluginJarDirectory = "plugins"
            + System.getProperty("file.separator");

    Registry vmdRegistry;

    RemoteServer server;

    User user;

    List<PluginDescriptor> availablePlugins;

    List<PluginDescriptor> localPlugins = new ArrayList<PluginDescriptor>();

    List<ClientPlugin> plugins = Collections
            .synchronizedList(new ArrayList<ClientPlugin>());

    ConnectionSustainer cs;

    private static final PluginDescriptor VmdDesktopDescriptor = new PluginDescriptor(
            "szymon", "", "VmdDesktop", "put.cs.vmd.desktop.client.VmdDesktop",
            "put.cs.vmd.desktop.server.VmdDesktopPlugin", true,
            "put.cs.vmd.desktop.server.VmdDesktopBuilder");

    public VMDClient() {
        File pluginDirectory = new File(pluginJarDirectory);
        if (!pluginDirectory.exists()) {
            if (!pluginDirectory.mkdir()) {
                System.out.println("Nie udalo sie utworzyc katalogu pluginow");
                System.exit(0);
            }
        } else if (pluginDirectory.isFile()) {
            System.out.println("Plik: " + pluginDirectory.getAbsolutePath()
                    + " blokuje utworzenie katalogu tymczasowego");
            System.exit(0);
        }
    }

    /**
     * @param args
     * @throws IllegalAccessException
     * @throws InstantiationException
     */
    @SuppressWarnings("unchecked")
    public static void main(String[] args) {
        final VMDClient client = new VMDClient();
        client.connectToServer();
        client.loginOnServer();

        try {
            client.availablePlugins = Arrays.asList(client.user
                    .listClientPlugins());
            CommonVariablesContainer.availablePlugins
                    .addAll(client.availablePlugins);
            CommonVariablesContainer.availablePlugins
                    .remove(VmdDesktopDescriptor);
        } catch (RemoteException e1) {
            System.out.println("nie powiodło się pobranie listy pluginów");
            e1.printStackTrace();
        }

        if (!client.availablePlugins.contains(VmdDesktopDescriptor)) {
            client.exit();
        }
        VmdDesktopInterface plugin = (VmdDesktopInterface) client
                .createClientPluginInstance(VmdDesktopDescriptor);

        // dodanie pluginów do menu desktopu
        for (PluginDescriptor descr : client.availablePlugins) {
            if (!descr.equals(VmdDesktopDescriptor)) {
                System.out.println(descr.name);
                plugin.addPlugin(descr);
            }
        }

        plugin.setVmdDesktopListener(new VmdDesktopListener() {
            public void closeDesktop() {
                client.exit();

            }

            public ClientPlugin preparePluginJar(PluginDescriptor descriptor) {
                return client.createClientPluginInstance(descriptor);
            }

        });
    }

    private ClientPlugin createClientPluginInstance(
            final PluginDescriptor descriptor) {

        if (!localPlugins.contains(descriptor)) {
            downloadPluginJar(descriptor);
            localPlugins.add(descriptor);
        }

        ClientPlugin plugin = ClientPluginLoader
                .createPlugin(descriptor.clientPackagePath);
        if (descriptor.isRemote) {
            RemoteAgent ownAgent = plugin.getOwnAgent();
            try {
                UnicastRemoteObject.exportObject(ownAgent);
            } catch (RemoteException e1) {
                System.out.println("nie udał sie exportObject");
                e1.printStackTrace();
            }
            try {
                RemoteAgent ra = user.getRemotePluginAgent(
                        descriptor.clientPackagePath, ownAgent);
                plugin.setForeignAgent(ra);
            } catch (RemoteException e) {
                e.printStackTrace();
            }
        }
        plugin.initInstance();
        synchronized (plugins) {
            plugins.add(plugin);
        }

        return plugin;
    }

    /**
     * @param user
     * @param descriptor
     */
    private void downloadPluginJar(PluginDescriptor descriptor) {
        FileOutputStream file = null;
        final String path = pluginJarDirectory + descriptor.name + ".jar";
              try {
                file = new FileOutputStream(path);
                file.write(user
                        .downloadClientPlugin(descriptor.clientPackagePath));

            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            } catch (RemoteException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    file.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        try {
            ClassPathLoader.addFile(path);
        } catch (IOException e2) {
            e2.printStackTrace();
        }
    }

    private void loginOnServer() {
        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 2));
        panel.add(new JLabel("login: "));
        JTextField login = new JTextField("szymon");
        panel.add(login);

        panel.add(new JLabel("hasło: "));
        JPasswordField pass = new JPasswordField("haslo");
        panel.add(pass);
        int answer = JOptionPane.showOptionDialog(null, panel, "Łączenie...",
                JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, null, null);
        if (answer == JOptionPane.CANCEL_OPTION) {
            System.exit(0);
        }
        if (answer == JOptionPane.OK_OPTION) {
            user = getUser(login.getText(), new String(pass.getPassword()));
            if (user == null) {
                System.out.println("logowanie nie powiodlo sie");
                System.exit(0);
            }
            cs = new ConnectionSustainer();
            cs.setUser(user);
            cs.setInterval(4000);
            cs.enable();
            try {
            	
                CommonVariablesContainer.rff = user.getUserFileSystem();
                CommonVariablesContainer.fsListener = new FsChangeListenerServer();
                UnicastRemoteObject
                        .exportObject(CommonVariablesContainer.fsListener);
                user.setFsChangeListener(CommonVariablesContainer.fsListener);
                if (CommonVariablesContainer.rff == null)
                    throw new RemoteException();
            } catch (RemoteException e) {

                e.printStackTrace();
                System.out
                        .println("Nie udalo sie pobranie zdalnej fabryki plików");
            }

        }
    }

    private RemoteServer getRemoteServer(String adress, int port) {
        RemoteServer server = null;

        ClientTwoWaySocketFactory fac = null;
        fac = new ClientTwoWaySocketFactory();
        try {
            RMISocketFactory.setSocketFactory(fac);
            fac.establishSignallingChannel(adress, port);
        } catch (IOException e) {
            System.out.println("Cannot establish sygnalling");
            return null;
        }

        try {
            vmdRegistry = LocateRegistry.getRegistry(adress, port, fac);
        } catch (RemoteException e) {
            System.out.println("Cannot get remote registry");
            return null;
        }
        try {
            server = (RemoteServer) vmdRegistry.getRemoteServer();
        } catch (RemoteException e) {
            System.out.println("Cannot get remoteServer from registry");
            return null;
        } catch (NotBoundException e) {
            System.out.println("Remote Server is not bound");
            return null;
        }
        return server;
    }

    private User getUser(String login, String pass) {
        User user = null;
        try {
            user = server.login(login, pass);
            
         } catch (RemoteException e) {
            System.out.println("Cannot get user from server:" + e.getMessage());
        }
        return user;

    }

    protected void connectToServer() {

        JPanel panel = new JPanel();
        panel.setLayout(new GridLayout(2, 2));
        panel.add(new JLabel("adres: "));
        JTextField adres = new JTextField("localhost");
        panel.add(adres);

        panel.add(new JLabel("port: "));
        JTextField port = new JTextField("1410");
        panel.add(port);
        int answer = JOptionPane.showOptionDialog(null, panel, "Łączenie...",
                JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, null, null);
        if (answer == JOptionPane.CANCEL_OPTION) {
            System.exit(0);
        }
        if (answer == JOptionPane.OK_OPTION) {
            this.server = getRemoteServer(adres.getText(), Integer
                    .parseInt(port.getText()));
        }
        if (this.server == null)
            System.exit(0);
    }

    private void exit() {
        int answer = JOptionPane.showOptionDialog(null,
                "Czy usunąć lokalne pliki?", "Koniec pracy",
                JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
                null, null, null);
        if (answer == JOptionPane.OK_OPTION) {
            for (PluginDescriptor plugin : localPlugins) {
                File file = new File(pluginJarDirectory + plugin.name + ".jar");
                file.delete();
            }
        }

        synchronized (plugins) {
            for (ClientPlugin plugin : plugins) {
                plugin.disposePlugin();
            }
        }
        try {
            cs.disable();
            user.logout();
        } catch (RemoteException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        System.exit(0);
    }

}
