package put.cs.vmd.server;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.rmi.RemoteException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import put.cs.vmd.gui.GUIServer;
import put.cs.vmd.server.io.CsvTypeIO;
import put.cs.vmd.server.io.CsvUserIO;
import put.cs.vmd.server.io.VmdLogWriter;
import put.cs.vmd.server.plugins.PluginManager;
import put.cs.vmd.server.usr.VmdLoggedUsersManager;
import twcsckernel.serverKernel.impl.RemoteServerImpl;
import twcsckernel.serverKernel.io.LogWriter;
import twcsckernel.serverKernel.utils.RegisteredTypes;
import twcsckernel.serverKernel.utils.RegisteredUsers;
import twcsckernel.serverKernel.utils.RmiManager;
import twcsckernel.serverKernel.utils.ServerGlobalState;
import twcsckernel.serverKernel.utils.UserData;

public class VMDserver {

	private static VMDserver vmdServer = new VMDserver();

	public static final int DEFAULT_CONNECTION_TIMEOUT = 2000;
	public static final int DEFAULT_GARBAGE_INTERVAL = 600000; // 10 min.
	public static final int DEFAULT_PORT = 1410; // :]
	public boolean ifGUI = false;

	public static final String DEFAULT_PLUGINS_PATH = "Plugins";
	public static final String DEFAULT_USERS_CSV = "users.csv";
	public static final String DEFAULT_TYPES_CSV = "types.csv";

	private static final String SHUT_DOWN_MESSAGE = "Server shut down successfully";
	private static final String START_MESSAGE = "VMD server successfully started.";

	private static IllegalArgumentException oneParam = new IllegalArgumentException(
			"Ten przełącznik wymaga parametru");
	private static IllegalArgumentException noParams = new IllegalArgumentException(
			"Ten przełącznik nie wymaga parametru");

	

	private Map<String, Command> cmdMap = initCommands();
	/* *********** parametry do ustawienia przez komendy***** */
	int sharedListenerType;
	int port;
	int connectionTimeout;

	long garbageInterval;

	String regUsersPath;
	String regTypesPath;

	String pluginsDirectoryPath;

	PrintStream psUser;
	PrintStream psError;
	PrintStream psInfo;

	private RegisteredUsers registeredUsers;
	private RegisteredTypes registeredTypes;
	private RmiManager rmiManager;
	private PluginManager pluginManager;
	private ServerGlobalState globalState;

	public static void main(String[] argv) {

		if (argv.length == 1 && argv[0].equals("--help")) {
			vmdServer.writeHelp();
			return;
		}

		vmdServer.port = DEFAULT_PORT;
		vmdServer.connectionTimeout = DEFAULT_CONNECTION_TIMEOUT;
		vmdServer.sharedListenerType = VmdLoggedUsersManager.USE_USER_SHARED_LISTENER;
		vmdServer.garbageInterval = DEFAULT_GARBAGE_INTERVAL;
		vmdServer.regTypesPath = DEFAULT_TYPES_CSV;
		vmdServer.regUsersPath = DEFAULT_USERS_CSV;
		vmdServer.pluginsDirectoryPath = DEFAULT_PLUGINS_PATH;
		vmdServer.psUser = System.out;
		vmdServer.psError = System.out;
		vmdServer.psInfo = System.out;

		for (String param : argv) {
			String[] valParam = param.split("=");
			Command cmd = null;
			try {
				if (valParam.length == 2) {
					cmd = vmdServer.cmdMap.get(valParam[0]);
					if (cmd != null)
						cmd.execute(valParam[1]);
				} else if (valParam.length == 1) {
					cmd = vmdServer.cmdMap.get(param);
					if (cmd != null)
						cmd.execute(null);
				}
			} catch (Exception e) {
				System.out
						.println("An error occured while processing parameter: '"
								+ param + "':\n" + e.getMessage());
				return;
			}
			if (cmd == null) {
				System.out.println("Incorrect parameter: '" + param
						+ "'.\nType --help for help.");
				return;
			}
		}
		if (!vmdServer.ifGUI)
		vmdServer.start();
	
	}

	private Map<String, Command> initCommands() {
		Map<String, Command> result = new HashMap<String, Command>();
		Class[] commands = this.getClass().getClasses();
		Command cmd;
		for (Class cmdClass : commands) {
			try {
				if (cmdClass.getName().indexOf("CmdSet") >= 0) {
					cmd = (Command) cmdClass.newInstance();
					result.put(cmd.getName(), cmd);
				}
			} catch (Exception e) {
			}
		}
		return result;
	}

	private void writeHelp() {
		System.out.println("Available switches:");
		Collection<Command> cmdSet = cmdMap.values();
		for (Command cmd : cmdSet)
			System.out.println(cmd.getName() + " - " + cmd.getDescription());
	}

	public void start() {

		try {
			registeredUsers = new RegisteredUsers(new CsvUserIO(regUsersPath));
			registeredTypes = new RegisteredTypes(new CsvTypeIO(regTypesPath));
			checkUserTypesCompatibility();
		} catch (Exception e) {
			System.out.println("cannot load configuration: " + e.getMessage());
			shutdown(1);
		}

		try {
			rmiManager = new RmiManager(port, true, connectionTimeout);
			rmiManager.setUserCollectorInterval(garbageInterval);
		} catch (IOException e) {
			System.out.println("Cannot create socketfactory or start server");
			shutdown(2);
		}

		LogWriter logWriter = new VmdLogWriter(psUser, psError, psInfo);

		try {
			pluginManager = new PluginManager(pluginsDirectoryPath,
					registeredTypes, logWriter);
		} catch (IOException e) {
			System.out.println("cannot create pluginManager");
			shutdown(3);
		}
		globalState = new ServerGlobalState(new VmdLoggedUsersManager(
				VmdLoggedUsersManager.USE_USER_SHARED_LISTENER), pluginManager
				.getPluginHandleContainer(), registeredTypes, registeredUsers,
				logWriter);

		try {
			rmiManager.startRegistryServer(new RemoteServerImpl(globalState,
					rmiManager));
		} catch (RemoteException e) {
			System.out.println("Cannot export RemoteServer");
			shutdown(4);
		}
		console();
	}

	private void console() {
		globalState.logWriter.writeInfoLog(START_MESSAGE);
		try {
			while (System.in.read() != 13) {
				shutdown(0);
			}
		} catch (IOException e) {
			globalState.logWriter.writeErrorLog(SHUT_DOWN_MESSAGE,
					LogWriter.ERROR_UNEXPECTED_EXCEPTION);
			shutdown(5);
		}
	}

	private void checkUserTypesCompatibility() throws Exception {
		UserData[] users = registeredUsers.getRegisteredUsers();
		Set<String> types = registeredTypes.getTypeNamesSet();
		for (UserData user : users)
			if (!types.contains(user.getType()))
				throw new Exception("Undefined user type for user: "
						+ user.getLogin());
	}

	public void shutdown(int code) {
		rmiManager.stopRegistryServer();
		if (code == 0)
			globalState.logWriter.writeInfoLog(SHUT_DOWN_MESSAGE);
		else
			globalState.logWriter.writeInfoLog("Server shut down, error code: "
					+ code);
		if (psUser != System.out)
			psUser.close();
		if (psError != System.out)
			psError.close();
		if (psInfo != System.out)
			psInfo.close();
		System.exit(code);
	}

	public static class CmdSetPort implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.port = Integer.parseInt(param);
		}

		public String getName() {
			return "port";
		}

		public String getDescription() {
			return "ustawia port serwera, np. port=1410";
		}

	}

	public static class CmdSetGarbageInterval implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.garbageInterval = Integer.parseInt(param);
		}

		public String getName() {
			return "gc";
		}

		public String getDescription() {
			return "ustawia interwal user garbage collectora(ms), np. gc=6000";
		}

	}
	public static interface Command {
		void execute(String param) throws Exception;
		String getName();
		String getDescription();
	}

	public static class CmdSetConnectionTimeout implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.connectionTimeout = Integer.parseInt(param);
		}

		public String getName() {
			return "ct";
		}

		public String getDescription() {
			return "ustawia timeout dla proby polaczenia pasywnego(ms),np. ct=2000";
		}

	}

	public static class CmdSetSharedListenerType implements Command {

		private static IllegalArgumentException notAllowed = new IllegalArgumentException(
				"Allowed values: user/global");

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			if (param.equals("global"))
				vmdServer.sharedListenerType = VmdLoggedUsersManager.USE_GLOBAL_SHARED_LISTENER;
			else if (param.equals("user"))
				vmdServer.sharedListenerType = VmdLoggedUsersManager.USE_USER_SHARED_LISTENER;
			else
				throw notAllowed;

		}
		public String getName() {
			return "shl";
		}

		public String getDescription() {
			return "ustawia rodzaj wspoldzielonego listenera (global/user), np. shl=user";
		}

	}

	public static class CmdSetInterfaceType implements Command {

		public void execute(String param) throws Exception {
			if (param != null)
				throw noParams;
				GUIServer gui = new GUIServer(vmdServer.cmdMap,vmdServer);
				gui.newFrame();
				vmdServer.ifGUI=true;
		}

		public String getName() {
			return "gui";
		}

		public String getDescription() {
			return "wskazuje na uruchomienie graficznego edytora parametrów serwera";
		}

	}

	public static class CmdSetRegTypesPath implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.regTypesPath = param;
		}

		public String getName() {
			return "tpath";
		}

		public String getDescription() {
			return "ustawia sciezke do pliku zarejestrowanych typow, np. tpath=types.csv";
		}

	}

	public static class CmdSetRegUsersPath implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.regUsersPath = param;
		}

		public String getName() {
			return "upath";
		}

		public String getDescription() {
			return "ustawia sciezke do pliku zarejestrowanych uzytkownikow, np. upath=users.csv";
		}

	}

	public static class CmdSetPluginDirPath implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.pluginsDirectoryPath = param;
		}

		public String getName() {
			return "plugins";
		}

		public String getDescription() {
			return "ustawia katalog pluginow, np. plugins=/etc/Plugins";
		}

	}

	public static class CmdSetUserLog implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.psUser = new PrintStream(
					new FileOutputStream(param, true));
		}

		public String getName() {
			return "ulog";
		}

		public String getDescription() {
			return "ustawia plik logu uzytkownikow, np. ulog=user.log";
		}

	}

	public static class CmdSetErrorLog implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.psError = new PrintStream(new FileOutputStream(param,
					true));
		}

		public String getName() {
			return "elog";
		}

		public String getDescription() {
			return "ustawia plik logu bledow, np. elog=error.log";
		}

	}

	public static class CmdSetInfoLog implements Command {

		public void execute(String param) throws Exception {
			if (param == null)
				throw oneParam;
			vmdServer.psInfo = new PrintStream(
					new FileOutputStream(param, true));
		}

		public String getName() {
			return "ilog";
		}

		public String getDescription() {
			return "ustawia plik logu info, np. ilog=info.log";
		}

	}
}
