package put.cs.vmd.server.io;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import twcsckernel.serverKernel.io.TypeIO;
import twcsckernel.serverKernel.utils.TypeData;

/**
 * Czytnik zarejestrowanych typów użytkowników z pliku CSV.
 * 
 * @author VMD Group
 * 
 */
public class CsvTypeIO implements TypeIO {
	String path = null;

	private static final String regex = ";";

	/**
	 * @param path -
	 *            ścieżka do pliku CSV
	 */
	public CsvTypeIO(String path) {
		this.path = path;
	}

	public Map<String, TypeData> readTypes() throws IOException {
		HashMap<String, TypeData> typeMap = new HashMap<String, TypeData>();
		BufferedReader input = null;
		input = new BufferedReader(new FileReader(path));

		String line = null;
		try {
			while ((line = input.readLine()) != null) {
				TypeData type = typeDataDeserialize(line);
				if (type != null)
					typeMap.put(type.getTypeName(), type);
			}
		} finally {
			try {
				input.close();
			} catch (Exception e) {

			}
		}

		return typeMap;
	}

	public boolean addNewType(TypeData typeData) throws IOException {
		PrintWriter output = null;
		try {
			output = new PrintWriter(new FileWriter(path, true));
		} catch (IOException e) {
			return false;
		}
		output.println(typeDataSerialize(typeData));
		output.close();
		return true;
	}

	public boolean removeType(String name, Map<String, TypeData> types)
			throws IOException {
		if (!types.containsKey(name))
			return false;
		PrintWriter output = null;
		try {
			output = new PrintWriter(new FileWriter(path));
		} catch (IOException e) {
			return false;
		}
		types.remove(name);
		Iterator iterator = types.keySet().iterator();
		while (iterator.hasNext()) {
			TypeData type = types.get(iterator.next());
			output.println(typeDataSerialize(type));
		}
		output.close();
		return true;
	}

	private String typeDataSerialize(TypeData type) {
		StringBuffer strBuf = new StringBuffer();
		String[] paths = type.getPluginDirNames();

		strBuf.append(type.getTypeName());
		for (String path : paths) {
			strBuf.append(";");
			strBuf.append(path);
		}

		return strBuf.toString();
	}

	private TypeData typeDataDeserialize(String line) {
		String[] items = line.split(regex);
		if (items.length == 0)
			return null;
		TypeData type = new TypeData();
		type.setTypeName(items[0]);
		for (int i = 1; i < items.length; i++)
			type.addPluginPath(items[i]);
		return type;
	}

}
