package put.cs.vmd.server.io;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import twcsckernel.serverKernel.io.UserIO;
import twcsckernel.serverKernel.utils.UserData;

/**
 * Czytnik zarejestrowanych użytkowników z pliku CSV.
 * 
 * @author VMD Group
 * 
 */
public class CsvUserIO implements UserIO {
	String path = null;

	private static final String regex = ";";

	/**
	 * @param path -
	 *            ścieżka do pliku CSV.
	 */
	public CsvUserIO(String path) {
		this.path = path;
	}

	public Map<String, UserData> readUsers() throws IOException {
		HashMap<String, UserData> userMap = new HashMap<String, UserData>();
		BufferedReader input = null;
		input = new BufferedReader(new FileReader(path));

		String line = null;
		try {
			while ((line = input.readLine()) != null) {
				UserData user = userDataDeserialize(line);
				if (user != null)
					userMap.put(user.getLogin(), user);
			}
		} finally {
			try {
				input.close();
			} catch (Exception e) {

			}
		}

		return userMap;
	}

	public boolean addNewUser(UserData user) throws IOException {
		PrintWriter output = null;
		try {
			output = new PrintWriter(new FileWriter(path, true));
		} catch (IOException e) {
			return false;
		}
		output.println(userDataSerialize(user));
		output.close();
		return true;

	}

	public boolean removeUser(String login, Map<String, UserData> logins)
			throws IOException {
		if (!logins.containsKey(login))
			return false;
		PrintWriter output = null;
		try {
			output = new PrintWriter(new FileWriter(path));
		} catch (IOException e) {
			return false;
		}
		logins.remove(login);
		Iterator iterator = logins.keySet().iterator();
		while (iterator.hasNext()) {
			UserData user = logins.get(iterator.next());
			output.println(userDataSerialize(user));
		}
		output.close();
		return true;

	}

	private String userDataSerialize(UserData user) {
		return user.getLogin() + ";" + user.getEncodedPasswd() + ";"
				+ user.getName() + ";" + user.getType() + ";" + user.getRoot();
	}

	private UserData userDataDeserialize(String line) {
		String[] items = line.split(regex);
		if (items.length != 5)
			return null;
		// format pliku login;haszowaneHaslo;nazwa;typUzytkownika;root

		return new UserData(items[0], items[1], items[2], items[3], items[4]);
	}

}
