package put.cs.vmd.server.io;

import java.io.File;
import java.io.IOException;

import twcsckernel.projectbase.io.FileSecurityManager;
import twcsckernel.projectbase.io.UserRootPathManager;
import twcsckernel.serverKernel.utils.UserData;

public class RootPathSecurityManager implements FileSecurityManager {

	private String rootPath;

	private UserRootPathManager pathManager;

	public RootPathSecurityManager(UserData userData) throws IOException {
		pathManager = new UserRootPathManager();
		pathManager.setUserRootPath(userData.getRoot());
		rootPath = pathManager.getRootPath();
	}

	public int checkReadAccess(File file) {
		if (checkRootPath(file) == ACC_OK) {
			if (file.exists())
				return ACC_OK;
			else
				return ACC_NOT_FOUND;
		}
		return ACC_ILLEGAL;
	}

	public int checkWriteAccess(File file) {
		if (checkRootPath(file) == ACC_OK)
			return ACC_OK;
		return ACC_ILLEGAL;
	}

	public int checkRootPath(File file) {
		String path = null;
		try {
			path = file.getCanonicalPath();
		} catch (IOException e) {
			return ACC_ILLEGAL;
		}
		if (path.startsWith(rootPath))
			return ACC_OK;
		return ACC_ILLEGAL;
	}

	public UserRootPathManager getUserRootPathManager() {
		return pathManager;
	}

	/*
	 * ścieżka Roota musi być ścieżką kanoniczną
	 */
	public void setRootPath(String userRootPath) throws IOException {
		pathManager.setUserRootPath(userRootPath);
		rootPath = pathManager.getRootPath();
	}

}
