package put.cs.vmd.server.io;

import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import twcsckernel.serverKernel.io.LogWriter;

/**
 * 
 * @author VMD Group
 * 
 */
public class VmdLogWriter implements LogWriter {

	private static final String LOG_TERM = "*------------------------------------*";

	private PrintStream userLog;

	private PrintStream errorLog;

	private PrintStream infoLog;
	
	private DateFormat dfmt=new SimpleDateFormat("HH:mm:ss (dd.MM.yyyy)");
	
	private Date currentTime=new Date();

	public VmdLogWriter(PrintStream userLog, PrintStream errorLog,
			PrintStream infoLog) {
		this.errorLog = errorLog;
		this.infoLog = infoLog;
		this.userLog = userLog;
	}

	public void writeUserLog(String message, int type) {
		if (userLog != null)
			synchronized (userLog) {
				switch (type) {
					case LogWriter.USER_LOGIN :
						userLog
								.println("***************LOGIN*****************");
						break;
					case LogWriter.USER_REMOTE_LOGOUT :
						userLog
								.println("***********REMOTE LOGOUT*************");
						break;
					case LogWriter.USER_KICK_LOGOUT :
						userLog
								.println("************KICK LOGOUT**************");
						break;
					default :
						userLog
								.println("*******UNKNOWN LOGIN LOG TYPE********");
						break;
				}
				userLog.println(getTime());
				userLog.println(message);
				userLog.println(LOG_TERM);
			}
	}

	public void writeErrorLog(String message, int type) {
		if (errorLog != null)
			synchronized (errorLog) {
				switch (type) {
					case LogWriter.ERROR_LOGIN_PASSWORD :
						errorLog
								.println("*********ERROR LOGIN PASSWORD********");
						break;
					case LogWriter.ERROR_PLUGIN_LOADING :
						errorLog
								.println("********ERROR PLUGIN LOADING*********");
						break;
					case LogWriter.ERROR_UNEXPECTED_EXCEPTION :
						errorLog
								.println("******ERROR UNEXPECTED EXCEPTION*****");
						break;
					case LogWriter.ERROR_OTHER :
						errorLog
								.println("*************OTHER ERROR*************");
						break;
					default :
						errorLog
								.println("*******UNKNOWN ERROR LOG TYPE********");
						break;
				}
				errorLog.println(getTime());
				errorLog.println(message);
				errorLog.println(LOG_TERM);
			}
	}

	public void writeInfoLog(String message) {
		if (infoLog != null)
			synchronized (infoLog) {
				infoLog.println("**************** INFO ****************");
				infoLog.println(getTime());
				infoLog.println(message);
				infoLog.println(LOG_TERM);
			}
	}

	private String getTime() {
		currentTime.setTime(System.currentTimeMillis());
		return dfmt.format(currentTime);
	}

}
