package put.cs.vmd.server.io;

import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import twcsckernel.projectbase.common.FsChangeListener;
import twcsckernel.projectbase.io.FsChangeDescriptor;
import twcsckernel.projectbase.io.UserRootPathManager;
import twcsckernel.serverKernel.impl.UserImpl;
import twcsckernel.serverKernel.io.SharedFsChangeListener;
import twcsckernel.serverKernel.utils.RootAdjuster;

/**
 * Klasa
 * 
 * @author VMD Group
 * 
 */
public class VmdSharedFsChangeListener implements SharedFsChangeListener {
	private Set<ParamsBind> paramBindSet = new HashSet<ParamsBind>();

	private Map<UserImpl, Set<ParamsBind>> paramBindMap = new HashMap<UserImpl, Set<ParamsBind>>();

	private UserRootPathManager rootManager;

	private static class ParamsBind {
		public final UserRootPathManager rootManager;

		public final FsChangeListener changeListener;

		public ParamsBind(FsChangeListener listener, UserRootPathManager manager) {
			changeListener = listener;
			rootManager = manager;
		}
	}

	public VmdSharedFsChangeListener(UserRootPathManager rootManager) {
		this.rootManager = rootManager;
	}

	public void fsChangeNotify(FsChangeDescriptor fsDescriptor) {
		if (rootManager == null)
			for (ParamsBind pBind : paramBindSet)
				try {
					FsChangeDescriptor tmpDescr = RootAdjuster.adjustRootPath(
							fsDescriptor, pBind.rootManager);
					if (tmpDescr != null)
						pBind.changeListener.fsChangeNotify(tmpDescr);
				} catch (RemoteException e) {
				}
		else {
			FsChangeDescriptor tmpDescr = RootAdjuster.adjustRootPath(
					fsDescriptor, rootManager);
			for (ParamsBind pBind : paramBindSet)
				try {
					pBind.changeListener.fsChangeNotify(tmpDescr);
				} catch (RemoteException e) {
				}
		}
	}

	public void addFsChangeListener(FsChangeListener fsChL, UserImpl owner,
			UserRootPathManager rootManager) {
		Set<ParamsBind> userParamBindSet = paramBindMap.get(owner);
		if (userParamBindSet == null) {
			userParamBindSet = new HashSet<ParamsBind>();
			paramBindMap.put(owner, userParamBindSet);
		}
		ParamsBind newBind = new ParamsBind(fsChL, rootManager);
		userParamBindSet.add(newBind);
		paramBindSet.add(newBind);
	}

	public boolean removeFsChangeListener(FsChangeListener fsChL, UserImpl owner) {
		Set<ParamsBind> userParamBindSet = paramBindMap.get(owner);
		if (userParamBindSet == null)
			return false;
		for (ParamsBind paramBind : userParamBindSet)
			if (paramBind.changeListener.equals(fsChL)) {
				userParamBindSet.remove(paramBind);
				paramBindSet.remove(paramBind);
				return true;
			}
		return false;
	}

	public void removeUserListeners(UserImpl owner) {
		Set<ParamsBind> userParamBindSet = paramBindMap.get(owner);
		if (userParamBindSet == null)
			return;
		paramBindSet.removeAll(userParamBindSet);
		paramBindMap.remove(owner);
	}
}
